package com.persagy.fm.sop.controller;

import com.alibaba.fastjson.JSONObject;
import com.persagy.fm.common.response.FmResponseMsg;
import com.persagy.fm.common.response.FmResponseUtil;
import com.persagy.fm.sop.model.dto.RefSopInfo;
import com.persagy.fm.sop.model.vo.*;
import com.persagy.fm.sop.service.GroupSopService;
import com.persagy.fm.sop.service.SopEditService;
import com.persagy.old.common.CommonConst;
import com.persagy.old.common.ToolsUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

@RestController
@RequestMapping("restGroupSopService")
public class GroupSopController {

    @Autowired
    private GroupSopService groupSopService;


    /**
     * 集团sop-新增页-添加发布状态sop
     * <p>
     * 重构完成
     *
     * @param saveSopVo
     * @return
     * @throws Exception
     */
    @PostMapping("/addPublishedSop")
    public FmResponseMsg addPublishedSop(@RequestBody SaveSopVo saveSopVo) throws Exception {
        //设置查询的项目类型为集团类型的
        saveSopVo.setProject_id(CommonConst.group_sop_id);
        groupSopService.addPublishedSop(saveSopVo);
        return FmResponseUtil.successMsg("保存成功");
    }


    /**
     * 集团sop-新增页-验证sop
     * <p>
     * 重构完成
     *
     * @param saveSopVo
     * @return
     * @throws Exception
     */
    @PostMapping("/verifySopBeforePublish")
    public FmResponseMsg verifySopBeforePublish(@RequestBody SaveSopVo saveSopVo) throws Exception {
        saveSopVo.setProject_id(CommonConst.group_sop_id);
        groupSopService.verifySopForPublish(saveSopVo);
        return FmResponseUtil.successMsg("验证成功");
    }

    /**
     * 集团sop-新增页-添加草稿状态sop
     * <p>
     * 重构完成
     *
     * @param
     * @return
     * @throws Exception
     */
    @PostMapping("/addDraftSop")
    public String addDraftSop(@RequestBody SaveSopEditVo saveSopEditVo) throws Exception {
        saveSopEditVo.setProject_id(CommonConst.group_sop_id);
        String sopId = groupSopService.addDraftSop(saveSopEditVo);
        Map<String, Object> ret = new HashMap<>();
        ret.put("sop_id", sopId);
        return ToolsUtil.successJsonMsg(ret);
    }

    /**
     * 集团sop-编辑页-编辑sop时验证sop名称
     * 重构完成
     *
     * @param verifySopNameVo
     * @return
     * @throws Exception
     */
    @PostMapping("/verifySopNameForUpdate")
    public String verifySopNameForUpdate(@RequestBody @Validated VerifySopNameVo verifySopNameVo) throws Exception {
        boolean isCan = groupSopService.verifySopBeforePublish(verifySopNameVo);
        return ToolsUtil.successJsonMsg(JSONObject.parseObject("{\"can_use\":" + isCan + "}"));

    }

    /**
     * 详细页-查询当前sop被引用的sop列表
     * 重构完成
     *
     * @param querySopReferenceVo
     * @return
     * @throws Exception
     */

    @PostMapping("/queryReferencedSopListById")
    public String queryReferencedSopListById(@RequestBody QuerySopReferenceVo querySopReferenceVo) throws Exception {
        List<RefSopInfo> refSopInfos = groupSopService.queryReferencedSopListById(querySopReferenceVo);
        return ToolsUtil.successJsonMsg(refSopInfos);
    }

    /**
     * 新增页:查询可供选择的sop，复制、引用时用
     *
     * @param verifySopNameVo
     * @return
     * @throws Exception
     */
    @PostMapping("/verifySopNameForAdd")
    public String verifySopNameForAdd(@RequestBody VerifySopNameVo verifySopNameVo) throws Exception {
        boolean isCan = groupSopService.verifySopBeforePublish(verifySopNameVo);
        return ToolsUtil.successJsonMsg(JSONObject.parseObject("{\"can_use\":" + isCan + "}"));

    }


    /**
     * 集团sop-编辑页-保存草稿状态sop
     *
     * @param saveSopVo
     * @return
     * @throws Exception
     */
    @PostMapping("/saveDraftSop")
    public String saveDraftSop(@RequestBody SaveDraftSopVo saveSopVo) throws Exception {
        saveSopVo.setProject_id(CommonConst.group_sop_id);
        groupSopService.saveDraftSop(saveSopVo);
        return ToolsUtil.successJsonMsg("");
    }

    /**
     * 新增页:查询可供选择的sop，复制、引用时用
     * @param sopListForSelVo
     * @return
     * @throws Exception
     */
    @PostMapping("/querySopListForSel")
    public String querySopListForSel(@RequestBody QuerySopListForSelVo sopListForSelVo) throws Exception{
        sopListForSelVo.setProjectId(CommonConst.group_sop_id);
        JSONObject item = groupSopService.querySopListForSel(sopListForSelVo);
        return ToolsUtil.successJsonMsg(item);
    }

    /**
     * 集团sop-列表页  查询已发布的sop列表
     * @param sopListForSelVo
     * @return
     * @throws Exception
     */
    @PostMapping("/queryPublishedSopList")
    public String queryPublishedSopList(@RequestBody QuerySopListForSelVo sopListForSelVo) throws Exception{
        sopListForSelVo.setProjectId(CommonConst.group_sop_id);
        JSONObject item = groupSopService.queryPublishedSopList(sopListForSelVo);
        return ToolsUtil.successJsonMsg(item);
    }

    /**
     * 集团sop-列表页  查询已作废中的sop列表
     * @param queryHistoryListVo
     * @return
     * @throws Exception
     */
    @PostMapping("/queryDestroyedSopList")
    public String queryDestroyedSopList(@RequestBody QueryHistoryListVo queryHistoryListVo) throws Exception{
        queryHistoryListVo.setProjectId(CommonConst.group_sop_id);
        return groupSopService.queryDestroyedSopList(queryHistoryListVo).toString();
    }

    @Autowired
    private SopEditService sopEditService;
    /**
     * 集团sop-列表页 查询编辑中的sop列表
     * @param queryHistoryListVo
     * @return
     * @throws Exception
     */
    @PostMapping("/queryEditingSopList")
    public String queryEditingSopList(@RequestBody QueryHistoryListVo queryHistoryListVo) throws Exception{
        //根据
        queryHistoryListVo.setProjectId(CommonConst.group_sop_id);
        return groupSopService.queryEditingSopList(queryHistoryListVo).toJSONString();
    }

    /**
     * 集团sop-编辑页-发布sop
     * @param jsonStr
     * @return
     * @throws Exception
     */
    @PostMapping("/publishSop")
    public String publishSop(@RequestBody SaveSopVo saveSopVo) throws Exception{
        groupSopService.publishSop(saveSopVo);
        return ToolsUtil.successJsonMsg("发布成功");
    }

}
