package com.persagy.fm.sop.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.persagy.fm.common.exception.BusinessValidatorException;
import com.persagy.fm.sop.mapper.SopMapper;
import com.persagy.fm.sop.model.SopDTO;
import com.persagy.fm.sop.model.dto.*;
import com.persagy.fm.sop.model.vo.*;
import com.persagy.fm.sop.service.*;
import com.persagy.old.cache.OrderTypeCache;
import com.persagy.old.common.CommonConst;
import com.persagy.old.common.DateUtil;
import com.persagy.old.common.SopConst;
import com.persagy.old.common.StringUtil;
import com.persagy.old.dao.DBConst;
import com.persagy.old.dao.DBConst.Result;
import com.persagy.old.service.EquipModelServiceI;
import com.persagy.old.service.ObjectServiceI;
import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.BooleanUtils;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.lang.reflect.InvocationTargetException;
import java.util.*;

@Service
public class GroupSopServiceImpl implements GroupSopService {
    //@Autowired
    public ObjectServiceI objectService;

    @Autowired
    private SopService sopService;

    @Autowired
    private SopEditService sopEditService;

    @Autowired
    private SopSopRelService sopSopRelService;

    @Autowired
    private SopReviseMsgService sopReviseMsgService;

    //@Autowired
    private EquipModelServiceI equipModelService;
    //@Autowired
    private OrderTypeCache orderTypeCache;
    @Autowired
    private SopLabelRelService sopLabelRelService;
    @Autowired
    private SopObjRelService sopObjRelService;
    @Autowired
    private SopOrderTypeRelService sopOrderTypeRelService;
    @Autowired
    private SopEquipModelRelService sopEquipModelRelService;

   /* @Autowired
    private WoFlowPlanServiceI woFlowPlanService;
*/
    @Autowired
    private SopStateMachine sopStateMachine;

    @Autowired
    private SopHisService sopHisService;


    @Override
    public JSONObject queryPublishedSopList(QuerySopListForSelVo sopListForSelVo) throws Exception {
        String projectId = sopListForSelVo.getProjectId();
        //1.根据update和项目id查询
        List<Sop> sops = queryPublishedSopsByProjectIdAndUpdatetime(sopListForSelVo);
        /*
            //========2.移除待修订状态sop=======
            //如果不是集团的sop的查询
            if (!jsonObject.getString("project_id").equals(CommonConst.group_sop_id)) {
                //移除待修订z状态的sop
                removeGroupReviseSop(sops);
                String rootProjFuncType = objectService.queryRootProjFuncTypeById(jsonObject.getString("project_id"));
                removeWithoutProJFuncType(sops, rootProjFuncType);
            }
         */
        //=========3.筛选SOP========
        //根据 sign_codes 进行筛选
        sops = filterSopBySignCodes(sopListForSelVo.getSign_codes(), sops, projectId);
        sops = filterSopByFitScope(sopListForSelVo, sops);
        //=========4设置返回字段======
        //这里有两种类型的1-查询发布  2-查询可复制引用
        List<SopInfoVo> sopInfoVos = additionalSopDetail(sops,projectId,"1");
        //------------设置过滤项-----
        JSONObject item = new JSONObject();
        if (sopListForSelVo.isNeed_return_criteria()) {
            Map<String, Object> criteria = getSopListCriteria(sops, sopInfoVos);
            item.put("criteria", criteria);
        }
        //在此进行格式转化返回给前端
        item.put("content", sopInfoVos);
        item.put("count", sopInfoVos.size());
        return item;
    }

    @Override
    @Transactional
    public boolean addPublishedSop(SaveSopVo saveSopVo) throws Exception {

        //找到sop 与与对象、设备型号、工单类型、自定义标签、sop关系 并插入数据库中
        addSopRelation(saveSopVo);
        //落库本条sop*/
        //填充必要字段

        //将 请求体中的参数拷贝到我们里面
        //将对象映射到数据库实体上面去
        Sop sop = SopMapper.INSTANCE.SaveSopVoToSop(saveSopVo);
        /*sop.setSopStatus(SopConst.SOP_STATUS_PUBLISH);
        sop.setPublishStatus(SopConst.PUBLISH_STATUS_NORMAL);*/
        SopState sopState = sopStateMachine.newPublic();
        //设置状态
        setSopStatus(sop, sopState);
        sop.setVersionCount(1);
        sop.setVersion("V0.1");
        sop.setVersionExplain("[\""+ CommonConst.version_explain_create+"\"]");
        //将参数转化为指定的字段后进行存储
        sop.insert();
        return true;
    }

    private void setSopStatus(Sop sop, SopState sopState) {
        sop.setPublishStatus(sopState.getPublicStatus());
        sop.setSopStatus(sopState.getSopStatus());
    }

    @Override
    public boolean verifySopForPublish(SaveSopVo saveSopVo) throws Exception {
        String projectId = saveSopVo.getProject_id();
        List<String> objTypes = Arrays.asList("build","floor","space","system","shaft","equip");

        //验证对象内容是否有效
        //获取所有引用的对象
        List<String> objects = new ArrayList<>();
        List<SaveSopVo.FitObjsBean> fit_objs = saveSopVo.getFit_objs();
        if(CollectionUtils.isNotEmpty(fit_objs)) {
            for (SaveSopVo.FitObjsBean fit_obj : fit_objs) {
                if(objTypes.contains(fit_obj.getObj_type())) {
                    objects.add(fit_obj.getObj_id());
                }
            }
        }
        List<SopStepsBean> steps = saveSopVo.getSteps();
        if(CollectionUtils.isNotEmpty(steps)) {
            for (SopStepsBean step : steps) {
                List<SopStepsBean.StepContentBean> step_content = step.getStep_content();

                if(CollectionUtils.isNotEmpty(step_content)) {
                    for (SopStepsBean.StepContentBean stepContentBean : step_content) {
                        //content_objs
                        List<SopStepsBean.StepContentBean.ContentObjsBean> content_objs = stepContentBean.getContent_objs();
                        if(CollectionUtils.isNotEmpty(content_objs)) {
                            for (SopStepsBean.StepContentBean.ContentObjsBean content_obj : content_objs) {
                                if(objTypes.contains(content_obj.getObj_type())) {
                                    objects.add(content_obj.getObj_id());
                                }
                            }
                        }
                        //confirm_result
                        List<SopStepsBean.StepContentBean.ConfirmResultBean> confirm_result = stepContentBean.getConfirm_result();
                        if(CollectionUtils.isNotEmpty(confirm_result)) {
                            for (SopStepsBean.StepContentBean.ConfirmResultBean confirmResultBean : confirm_result) {
                                if(objTypes.contains(confirmResultBean.getObj_type())) {
                                    objects.add(confirmResultBean.getObj_id());
                                }
                            }
                        }
                    }
                }
            }
        }
        if(CollectionUtils.isNotEmpty(objects)) {
            String objectStr = objectService.queryBatchObj(JSONObject.toJSONString(objects), projectId, false);
            JSONObject objectJson = JSONObject.parseObject(objectStr);
            if(DBConst.Result.FAILURE.equals(objectJson.getString(Result.RESULT))){
                throw new Exception(objectStr);
            }
            Map<String, String> objectIdName = objectService.getObjectIdNameMap(objectStr);
            if(!objectIdName.isEmpty()) {
                Map<Object, Object> valiadtorData = new HashMap<>();
                for (String id : objectIdName.keySet()) {
                    valiadtorData.put(objectIdName.get(id),"添加的对象已作废,请前往编辑!");
                }
                throw new BusinessValidatorException(valiadtorData);
            }
        }
        //验证引用sop是否有效

        Set<String> sopRefSops = this.getSopRefSops(saveSopVo);
        if(CollectionUtils.isNotEmpty(sopRefSops)) {

            QueryWrapper<Sop> queryWrapper = new QueryWrapper<>();
            queryWrapper.in("sop_id",sopRefSops).eq("valid",false);
            //查询他所引用的sop 是否有无效的
            List<Sop> sops = sopService.list(queryWrapper);
            if(CollectionUtils.isNotEmpty(sops)) {

                Map<Object, Object> valiadtorData = new HashMap<>();
                for (Sop sop : sops) {
                    valiadtorData.put(sop.getSopName(),"引用的标准操作流程已被作废,请前往编辑!");
                }
                throw new BusinessValidatorException(valiadtorData);
            }
        }
        return true;
    }


    private boolean addSopRelation(SaveSopVo saveSopVo) throws Exception{

        //添加sop与对象关系-sop_obj_rel
        List<SaveSopVo.FitObjsBean> sopRelObjs = getSopRelObjs(saveSopVo);
        if(CollectionUtils.isNotEmpty(sopRelObjs)) {
            for (SaveSopVo.FitObjsBean sopRelObj : sopRelObjs) {

                SopObjRel sopObjRel = new SopObjRel();
                sopObjRel.setSopId(saveSopVo.getSop_id());
                sopObjRel.setObjId(sopRelObj.getObj_id());
                sopObjRel.setObjType(sopRelObj.getObj_type());
                sopObjRel.setValid(true);
                sopObjRel.insert();
            }
        }

        //添加sop与设备型号关系-sop_equip_model_rel
        List<String> equip_model_ids = saveSopVo.getEquip_model_ids();
        distinctCollection(equip_model_ids);
        if(CollectionUtils.isNotEmpty(equip_model_ids)) {
            for (String equip_model_id : equip_model_ids) {
                SopEquipModelRel sopEquipModelRel = new SopEquipModelRel();
                sopEquipModelRel.setEquipModelId(equip_model_id);
                sopEquipModelRel.setSopId(saveSopVo.getSop_id());
                sopEquipModelRel.setValid(true);
                sopEquipModelRel.insert();
            }
        }

        //添加sop与工单类型关系-sop_order_type_rel
        List<String> order_types = saveSopVo.getOrder_type();
        distinctCollection(order_types);
        if(CollectionUtils.isNotEmpty(order_types)) {
            for (String order_type : order_types) {
                SopOrderTypeRel sopOrderTypeRel = new SopOrderTypeRel();
                sopOrderTypeRel.setOrderType(order_type);
                sopOrderTypeRel.setProjectId(saveSopVo.getProject_id());
                sopOrderTypeRel.setValid(true);
                sopOrderTypeRel.setSopId(saveSopVo.getSop_id());
                sopOrderTypeRel.insert();
            }
        }
        //添加sop与自定义标签关系-sop_label_rel
        List<String> labels = saveSopVo.getLabels();
        distinctCollection(labels);
        if(CollectionUtils.isNotEmpty(labels)) {
            for (String label : labels) {
                SopLabelRel sopLabelRel = new SopLabelRel();
                sopLabelRel.setLabel(label);
                sopLabelRel.setSopId(saveSopVo.getSop_id());
                sopLabelRel.setValid(true);
                sopLabelRel.insert();
            }
        }
        //添加sop与sop关系-sop_sop_rel
        Set<String> sopRefSops = getSopRefSops(saveSopVo);
        if(CollectionUtils.isNotEmpty(sopRefSops)) {
            for (String sopRefSop : sopRefSops) {
                SopSopRel sopSopRel = new SopSopRel();
                sopSopRel.setProjectId(saveSopVo.getProject_id());
                sopSopRel.setRelSopId(sopRefSop);
                sopSopRel.setSopId(saveSopVo.getSop_id());
                sopSopRel.setValid(true);
            }
        }

        return true;

    }

    private void distinctCollection(List<String> list) {
        HashSet set = new HashSet(list);
        list.clear();
        list.addAll(set);
    }

    private  List<SaveSopVo.FitObjsBean> getSopRelObjs(SaveSopVo sop) throws InvocationTargetException, IllegalAccessException {
        List<SaveSopVo.FitObjsBean> sopRelObjs = new ArrayList<>();
        Map<String,SaveSopVo.FitObjsBean> objectId_Object = new HashMap<>();

        if(sop != null) {
            //查询适用的对象
            List<SaveSopVo.FitObjsBean> fit_objs = sop.getFit_objs();
            if(CollectionUtils.isNotEmpty(fit_objs)) {
                for (SaveSopVo.FitObjsBean fit_obj : fit_objs) {
                    objectId_Object.put(fit_obj.getObj_id(),fit_obj);
                }
            }
            List<SopStepsBean> steps = sop.getSteps();
            if(CollectionUtils.isNotEmpty(steps)) {
                for (SopStepsBean step : steps) {
                    //如果不是来自引用的对象
                    if(BooleanUtils.isFalse(step.isFrom_sop())) {
                        List<SopStepsBean.StepContentBean> step_content = step.getStep_content();
                        if(CollectionUtils.isNotEmpty(step_content)) {
                            for (SopStepsBean.StepContentBean stepContentBean : step_content) {
                                List<SopStepsBean.StepContentBean.ContentObjsBean> content_objs = stepContentBean.getContent_objs();
                                if(CollectionUtils.isNotEmpty(content_objs)) {
                                    for (SopStepsBean.StepContentBean.ContentObjsBean content_obj : content_objs) {
                                        //进行数值拷贝到一个对象中去
                                        SaveSopVo.FitObjsBean fitObjsBean = new SaveSopVo.FitObjsBean();
                                        BeanUtils.copyProperties(fitObjsBean,content_obj);
                                        objectId_Object.put(content_obj.getObj_id(),fitObjsBean);
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }

        //sop中 适用对象和步骤中的对象
        for (SaveSopVo.FitObjsBean value : objectId_Object.values()) {
            sopRelObjs.add(value);
        }
        return sopRelObjs;
    }

    public Set<String> getSopRefSops(SaveSopVo sop){
        LinkedHashSet<String> sopIds = new LinkedHashSet<>();
        if(sop != null) {

            List<SopStepsBean> steps = sop.getSteps();
            if(CollectionUtils.isNotEmpty(steps)) {
                for (SopStepsBean step : steps) {
                    if (BooleanUtils.isTrue(step.isFrom_sop())) {
                        sopIds.add(step.getSop_id());
                    } else {
                        List<SopStepsBean.StepContentBean> step_content = step.getStep_content();
                        if(CollectionUtils.isNotEmpty(step_content)) {
                            for (SopStepsBean.StepContentBean stepContentBean : step_content) {
                                if (BooleanUtils.isTrue(stepContentBean.isFrom_sop())) {
                                    sopIds.add(stepContentBean.getSop_id());
                                }

                            }
                        }
                    }
                }
            }
        }
        return sopIds;
    }

    //改造该方法使用业务上的数据

    @Override
    public String addDraftSop(SaveSopEditVo saveSopEditVo) throws Exception {
        Set<String> sopRefSops = this.getSopRefSops(saveSopEditVo);
        saveSopEditVo.setSop_edit_id(UUID.randomUUID().toString().replaceAll("-",""));

        //将表单提交的值拷贝过来
        SopEdit sopEdit = SopMapper.INSTANCE.SaveSopEditVoToSopEdit(saveSopEditVo);
        sopEdit.setRefSopIds(new ArrayList<>(sopRefSops));
        if (StringUtils.isBlank(sopEdit.getSopId())) {
            sopEdit.setSopId(UUID.randomUUID().toString().replaceAll("-", ""));
        }
        //sop类型 1 通用    2 项目专用
        if(CommonConst.group_sop_id.equals(saveSopEditVo.getProject_id())) {
            sopEdit.setSopType(SopConst.SOP_TYPE_GROUP);
        }else {
            sopEdit.setSopType(SopConst.SOP_TYPE_PROJECT);
        }
        if(StringUtils.isBlank(sopEdit.getSopStatus())) {
            //sop状态 0 草稿  1 已发布   2 已作废
            //则是新建的
            SopState sopState = sopStateMachine.saveDraft();
            setEditSopStatus(sopEdit, sopState);

        }else {
            SopState sopState = sopStateMachine.public2Draft();
            setEditSopStatus(sopEdit,sopState);
        }
        sopEdit.insert();
        return sopEdit.getSopId();
    }

    private void setEditSopStatus(SopEdit sopEdit, SopState sopState) {
        sopEdit.setSopStatus(sopState.getSopStatus());
        sopEdit.setPublishStatus(sopState.getPublicStatus());
    }

    public boolean insertRecord(SopEdit sopEdit) throws Exception {
        if (StringUtils.isBlank(sopEdit.getSopId())) {
            sopEdit.setSopId(UUID.randomUUID().toString().replaceAll("-", ""));
        }
        //sop类型 1 通用    2 项目专用
        sopEdit.setSopType(SopConst.SOP_TYPE_GROUP);

        if(StringUtils.isBlank(sopEdit.getSopStatus())) {
            //sop状态 0 草稿  1 已发布   2 已作废
            sopEdit.setSopStatus("0");
        }else {
            sopEdit.setSopStatus("1");
        }
        return sopEdit.insert();
    }

    @Override
    public boolean saveDraftSop(SaveDraftSopVo saveSopVo) throws Exception {
        String saveSopVoString = JSONObject.toJSONString(saveSopVo);
        saveSopVo.setProject_id(CommonConst.group_sop_id);
        String user_id = saveSopVo.getUser_id();
        if(StringUtils.isNotBlank(saveSopVo.getSop_id())) {
            //查询sop 待修订消息表
            //根据状态和 sop id 进行查询
            QueryWrapper<SopReviseMsg> queryWrapper = new  QueryWrapper<>();
            queryWrapper.eq("sop_id",saveSopVo.getSop_id()).eq("status","1");
            List<SopReviseMsg> list = sopReviseMsgService.list(queryWrapper);
            if(CollectionUtils.isNotEmpty(list)) {
                for (SopReviseMsg sopReviseMsg : list) {
                    String objId = sopReviseMsg.getObjId();
                    //如果不包含这个对象id
                    if(!saveSopVoString.contains(objId)) {
                        //设置为忽略状态
                        //根据msg_id 和sopid 为条件进行更新
                        sopReviseMsg.setStatus("2");
                        sopReviseMsg.updateById();
                    }
                }
            }

            //根据sopid 和 有效性查询编辑表
            QueryWrapper<SopEdit> queryBySopIdAndVaild = new QueryWrapper<>();
            queryBySopIdAndVaild.eq("sop_id",saveSopVo.getSop_id()).eq("valid",true);
            SopEdit sopEdit = sopEditService.getOne(queryBySopIdAndVaild);
            //数据库对象转化为业务对象
            SaveSopEditVo saveSopEditVo = SopMapper.INSTANCE.SopEditToSaveSopEditVo(sopEdit);
            //查询该改编辑表中引用的对象
            Set<String> sopRefSops = this.getSopRefSops(saveSopEditVo);


            SopEdit update = SopMapper.INSTANCE.SaveSopEditVoToSopEdit(saveSopVo);



            //将引用sop设置为sopRefSops
            update.setRefSopIds(new ArrayList<>(sopRefSops));
            if("1".equals(saveSopVo.getSop_status())) {
                //将状态更新为3  publish_status
                SopState sopState = sopStateMachine.public2Draft();
                setEditSopStatus(update,sopState);
            }

            if(sopEdit != null) {
                //更新sop  更新条件为sop_id  valid
                QueryWrapper<SopEdit> sopEditQueryWrapper = new QueryWrapper<>();
                sopEditQueryWrapper.eq("sop_id",sopEdit.getSopId()).eq("valid",true);
                sopEditService.update(update,sopEditQueryWrapper);

            }else {
                //插入编辑状态sop
                this.insertRecord(sopEdit);
            }
        }else {

            SopEdit data = SopMapper.INSTANCE.SaveSopEditVoToSopEdit(saveSopVo);
            //插入编辑状态sop
            this.insertRecord(data);
        }
        Sop sop = new Sop();
        sop.setSopId(saveSopVo.getSop_id());
        SopState sopState = sopStateMachine.public2Draft();
        setSopStatus(sop,sopState);
        return sop.updateById();
    }

    @Override
    public JSONObject querySopListForSel(QuerySopListForSelVo sopListForSelVo) throws Exception {
        String projectId = sopListForSelVo.getProjectId();
        String sopId = sopListForSelVo.getSop_id();
        String sop_name = sopListForSelVo.getSop_name();
        //是否返回包含大类对象的sop  true：返回   false：不返回 (默认返回)
        Boolean flag = true;
        if(BooleanUtils.isFalse(sopListForSelVo.isIs_contains_classObj())) {
            flag = false;
        }
        //1.根据update和项目id查询
        List<Sop> sops = queryPublishedSopsByProjectIdAndUpdatetime(sopListForSelVo);
        //2筛选sop

        /*
        removeReviseSop(sops);//已经直接放到sql 语句中查询了
        if (!jsonObject.getString("project_id").equals(CommonConst.group_sop_id)) {
			String rootProjFuncType = objectService.queryRootProjFuncTypeById(jsonObject.getString("project_id"));
			removeWithoutProJFuncType(sops, rootProjFuncType);
		}
         */
        removeRelSop(sops,projectId,sopId);

        if (!flag) {
            Iterator<Sop> iterator = sops.iterator();
            while (iterator.hasNext()) {
                Sop sopItem = iterator.next();
                if (isContainsClassObj(sopItem)) {
                    iterator.remove();
                }
            }
        }
        //=========3.筛选SOP========
        //根据 sign_codes 进行筛选
        sops = filterSopBySignCodes(sopListForSelVo.getSign_codes(), sops, projectId);


        //sops = filterSopByName(sop_name, sops); 直接在sql 中解决
        sops = filterSopByFitScope(sopListForSelVo, sops);


        //=========3设置返回字段======

        List<SopInfoVo> sopInfoVos = additionalSopDetail(sops,projectId,"2");

        JSONObject item = new JSONObject();
        if (sopListForSelVo.isNeed_return_criteria()) {
            Map<String, Object> criteria = getSopListCriteria(sops, sopInfoVos);
            item.put("criteria", criteria);
        }

        //JSONArray content = new JSONArray();
       /* for (Object object : sops) {
            content.add(JSONUtil.getJsonObjectWithKeys((JSONObject)object, querySopListForSel_keys));
            content.add(sopInfoVos
        }*/
        item.put("content", sopInfoVos);
        item.put("count", sopInfoVos.size());
        return item;
    }

    @Override
    public boolean publishSop(SaveSopVo saveSopVo) throws Exception {
        //验证必填项
        //根据id查询sop
        String sop_id = saveSopVo.getSop_id();
        Sop querySop = new Sop();
        Sop oldSop = querySop.selectById(sop_id);

        //转化为his sop  并填充必要的参数
        SopHis sopHis = new SopHis();

        org.springframework.beans.BeanUtils.copyProperties(oldSop, sopHis);
        //获取sop的版本信息
        Integer versionCount = oldSop.getVersionCount();
        Map<String, SopHis.RefedSopBean> stringProfileMap = proccessSopIdRefProfile(oldSop.getRefSopIds());
        //在每一步上追加该信息
        proccessSopItem(sopHis, stringProfileMap, null);
        //在refed中添加信息
        sopHis.setRefedSops(getSopProfileOfRefCurrentSop(sopHis.getSopId()));

        //将历史版本落库
        sopHisService.save(sopHis);

        //删除草稿
        sopEditService.removeById(saveSopVo.getSop_id());


        //更新sop
        Sop sop = SopMapper.INSTANCE.SaveSopVoToSop(saveSopVo);

        Set<String> sopRefSops = this.getSopRefSops(saveSopVo);

        sop.setRefSopIds(new ArrayList<>(sopRefSops));

        //首先更新相关的对象关系表
        updateSopRelation(saveSopVo);

        sop.setVersionExplain("[\"修订标准操作\"]");

        return updateRecordByKeyAndUpVersion(sop);
    }


    public boolean updateRecordByKeyAndUpVersion(Sop sopUpdate) throws Exception {
        Sop sop = new Sop();
        Sop sop1 = sop.selectById(sop.getSopId());
        Integer versionCount1 = sop1.getVersionCount();
        int count = versionCount1 + 1;
        sopUpdate.setVersion("V" + (double) count / 10);
        sopUpdate.setVersionCount(count);
        sopUpdate.updateById();
        return true;
    }





    private boolean updateSopRelation(SaveSopVo saveSopVo) throws Exception{

        //添加sop与对象关系-sop_obj_rel
        List<SaveSopVo.FitObjsBean> sopRelObjs = getSopRelObjs(saveSopVo);

        //查询数据库中的关联信息
        //删除所有
        sopObjRelService.deleteBySopId(saveSopVo.getSop_id());
        if(CollectionUtils.isNotEmpty(sopRelObjs)) {
            for (SaveSopVo.FitObjsBean sopRelObj : sopRelObjs) {
                SopObjRel sopObjRel = new SopObjRel();
                sopObjRel.setSopId(saveSopVo.getSop_id());
                sopObjRel.setObjId(sopRelObj.getObj_id());
                sopObjRel.setObjType(sopRelObj.getObj_type());
                sopObjRel.setValid(true);

                sopObjRel.insert();
            }
        }

        sopEquipModelRelService.deleteBySopId(saveSopVo.getSop_id());


        //添加sop与设备型号关系-sop_equip_model_rel
        List<String> equip_model_ids = saveSopVo.getEquip_model_ids();
        distinctCollection(equip_model_ids);
        if(CollectionUtils.isNotEmpty(equip_model_ids)) {
            for (String equip_model_id : equip_model_ids) {
                SopEquipModelRel sopEquipModelRel = new SopEquipModelRel();
                sopEquipModelRel.setEquipModelId(equip_model_id);
                sopEquipModelRel.setSopId(saveSopVo.getSop_id());
                sopEquipModelRel.setValid(true);
                sopEquipModelRel.insert();
            }
        }

        //删除所有

        sopOrderTypeRelService.deleteBySopId(saveSopVo.getSop_id());


        //添加sop与工单类型关系-sop_order_type_rel
        List<String> order_types = saveSopVo.getOrder_type();
        distinctCollection(order_types);
        if(CollectionUtils.isNotEmpty(order_types)) {
            for (String order_type : order_types) {
                SopOrderTypeRel sopOrderTypeRel = new SopOrderTypeRel();
                sopOrderTypeRel.setOrderType(order_type);
                sopOrderTypeRel.setProjectId(saveSopVo.getProject_id());
                sopOrderTypeRel.setValid(true);
                sopOrderTypeRel.setSopId(saveSopVo.getSop_id());
                sopOrderTypeRel.insert();
            }
        }
        //添加sop与自定义标签关系-sop_label_rel


        sopLabelRelService.deleteBySopId(saveSopVo.getSop_id());
        List<String> labels = saveSopVo.getLabels();
        distinctCollection(labels);
        if(CollectionUtils.isNotEmpty(labels)) {
            for (String label : labels) {
                SopLabelRel sopLabelRel = new SopLabelRel();
                sopLabelRel.setLabel(label);
                sopLabelRel.setSopId(saveSopVo.getSop_id());
                sopLabelRel.setValid(true);
                sopLabelRel.insert();
            }
        }



        sopSopRelService.deleteBySopId(saveSopVo.getSop_id());
        //添加sop与sop关系-sop_sop_rel
        Set<String> sopRefSops = getSopRefSops(saveSopVo);
        if(CollectionUtils.isNotEmpty(sopRefSops)) {
            for (String sopRefSop : sopRefSops) {
                SopSopRel sopSopRel = new SopSopRel();
                sopSopRel.setProjectId(saveSopVo.getProject_id());
                sopSopRel.setRelSopId(sopRefSop);
                sopSopRel.setSopId(saveSopVo.getSop_id());
                sopSopRel.setValid(true);
            }
        }

        return true;

    }




    /**
     * 处理引用当前sop的所有sop简介
     * @param sopId
     * @param sopId
     * @return
     * @throws Exception
     */
    public  List< SopHis.RefedSopBean>  getSopProfileOfRefCurrentSop( String sopId) throws Exception{
        //查询引用该sop的sopid
        QueryWrapper<SopSopRel> sopSopRelQueryWrapper = new QueryWrapper<>();
        sopSopRelQueryWrapper.eq("rel_sop_id",sopId);
        List<SopSopRel> list = sopSopRelService.list(sopSopRelQueryWrapper);
        List<String> refSopIds = new ArrayList<String>();
        for (SopSopRel sopSopRel : list) {
            refSopIds.add(sopSopRel.getSopId());
        }

        return  new ArrayList<>(proccessSopIdRefProfile(refSopIds).values());
    }



    /**
     * 处理sop详情 补充引用sop简介，添加内容对象变化标识
     * @param sop
     * @param refSopProfile
     * @Param changedObjIds     objId //对象变化数据key:objId value:{"objId":objId, "objName":objName, "objType":objType}
     * @return
     */
    public void proccessSopItem(SopHis sop, Map<String, SopHis.RefedSopBean> refSopProfile, Set<String> changedObjIds){
        if(CollectionUtils.isNotEmpty(changedObjIds)) {
            List<SopHis.FitObjsBean> fitObjs = sop.getFitObjs();
            if(CollectionUtils.isNotEmpty(fitObjs)) {
                for (SopHis.FitObjsBean fitObj : fitObjs) {
                    String obj_id = fitObj.getObj_id();
                    if(changedObjIds.contains(obj_id)) {
                        fitObj.setIs_revise(true);
                    }
                }
            }
            //處理步驟
            List<SopStepsBean> steps = sop.getSteps();
            for (SopStepsBean step : steps) {
                Boolean from_sop = step.isFrom_sop();
                if(BooleanUtils.isTrue(from_sop)) {
                    //将引用的简洁填充上去
                    String sop_id = step.getSop_id();
                    SopHis.RefedSopBean profile = refSopProfile.get(sop_id);
                    if(profile != null) {
                        step.setProfile(profile);
                    }
                }else {
                    List<SopStepsBean.StepContentBean> step_content = step.getStep_content();
                    if(CollectionUtils.isNotEmpty(step_content)) {
                        for (SopStepsBean.StepContentBean stepContentBean : step_content) {
                            Boolean from_sop1 = stepContentBean.isFrom_sop();
                            if(BooleanUtils.isTrue(from_sop1)) {
                                String sop_id = stepContentBean.getSop_id();
                                SopHis.RefedSopBean profile = refSopProfile.get(sop_id);
                                if(profile != null) {
                                    stepContentBean.setProfile(profile);
                                }
                            }
                        }
                    }
                }
            }
        }

    }

    /**
     * 处理引用的sop id与简介(sop_name,version,step_count,status_explain,update_time)
     * @Param userId        调用者
     * @param refSopIds    被引用sopId集合
     * @return map      key:sopId  value:sop对象
     * @throws Exception
     */
    public Map<String, SopHis.RefedSopBean> proccessSopIdRefProfile(List<String> refSopIds) throws Exception {
        List<Sop> sops = sopService.listByIds(refSopIds);
        Map<String, SopHis.RefedSopBean> sopId_Profile = new HashMap<>();
        for (Sop sopItem : sops) {
            SopHis.RefedSopBean profile = new SopHis.RefedSopBean();
            profile.setRef_sop_id(sopItem.getSopId());
            profile.setRef_sop_name(sopItem.getSopName());
            profile.setVersion(sopItem.getVersion());
            profile.setStep_count(sopItem.getSteps().size());
            profile.setStauts_explain(getSopStatus(sopItem));
            profile.setUpdate_time(DateUtil.transferDateFormat(sopItem.getUpdateTime(), CommonConst.date_format_save, CommonConst.date_format_show));
            sopId_Profile.put(sopItem.getSopId(), profile);
        }
        return sopId_Profile;
    }





    private Map<String, Object> getSopListCriteria(List<Sop> sops, List<SopInfoVo> sopInfoVos) throws Exception {
        //根据业务数据抽取条件
        Map<String,Object> criteria = getSopListCriteria(sops);

        //根据视图数据抽取条件
        //这个的我们使用视图对象进行抽取criteria.put("signs", signs);
        Map<String, Map<String,String>> signMap = getBathchMergeRecordBySign(sopInfoVos);
        List<Object> signs = new JSONArray();
//			signs.addAll(signMap.values());
        for (String signCode : signMap.keySet()) {
            signs.add(signMap.get(signCode));
        }
        criteria.put("signs", signs);
        return criteria;
    }



    private List<Sop> filterSopByFitScope(QuerySopListForSelVo sopListForSelVo, List<Sop> sops) throws Exception {
        JSONObject param = JSONObject.parseObject(JSONObject.toJSONString(sopListForSelVo));

        param.remove("project_id");
        JSONObject epuipModelsResJson = null, labelsResJson = null, orderTypeResJson = null, fitObjResJson = null;
        if (StringUtil.isEmptyList(param, "equip_model_ids")
                && StringUtil.isEmptyList(param, "labels")
                && StringUtil.isEmptyList(param, "order_type")
                && StringUtil.isEmptyList(param, "fit_obj_ids")) {
            return sops;
        }else {
            //查询条件后的交集
            Set<String> sopId = new HashSet<>();
            Map<String, Sop> sopMap = getSopMap(sops);
            //1.根据适用返回查询sopid
            //1.1根据设备型号查询
            if(!StringUtil.isEmptyList(param, "equip_model_ids")){

                List<SopEquipModelRel> list = sopEquipModelRelService.selectByEquipModelIds(sopListForSelVo.getEquip_model_ids());
                Set<String> tmp = new HashSet<>();
                for (SopEquipModelRel sopEquipModelRel : list) {
                    tmp.add(sopEquipModelRel.getSopId());
                }
                sopId.addAll(tmp);
            }
            //1.2根据自定义标签查询
            if( !StringUtil.isEmptyList(param, "labels")){
                List<SopLabelRel> list = sopLabelRelService.selectByLabels(sopListForSelVo.getLabels());
                Set<String> tmp = new HashSet<>();
                for (SopLabelRel sopLabelRel : list) {
                    tmp.add(sopLabelRel.getSopId());
                }
                sopId.retainAll(tmp);
            }
            //1.3根据工单类型查询
            if(!StringUtil.isEmptyList(param, "order_type")){
                List<SopOrderTypeRel> sopOrderTypeRels = sopOrderTypeRelService.selectByOrderTypes(sopListForSelVo.getOrder_type());
                Set<String> tmp = new HashSet<>();
                for (SopOrderTypeRel sopOrderTypeRel : sopOrderTypeRels) {
                    tmp.add(sopOrderTypeRel.getSopId());
                }
                sopId.retainAll(tmp);
            }
            //1.4根据适用对象查询
            if( !StringUtil.isEmptyList(param, "fit_obj_ids")){
                List<SopObjRel> sopObjRels = sopObjRelService.selectByObjectIds(sopListForSelVo.getFit_obj_ids());
                Set<String> tmp = new HashSet<>();
                for (SopObjRel sopObjRel : sopObjRels) {
                    tmp.add(sopObjRel.getSopId());
                }
                sopId.retainAll(tmp);
            }

            //查询出每张表中的id 然后求出一个交集来

            List<Sop> resultSops = new ArrayList<>();
            for (String s : sopId) {
                if (sopMap.containsKey(s)) {
                    resultSops.add(sopMap.get(sopId));
                }
            }
            return resultSops;
        }

    }

    /**
     *
     * Description:
     * @param sops
     * @return HashMap<String,Sop>    key:sop_id ; value: sop
     * @author yuecaipu
     * @since 2018年7月10日: 下午5:50:41
     * Update By yuecaipu 2018年7月10日: 下午5:50:41
     */
    private Map<String, Sop> getSopMap(List<Sop> sops) {
        Map<String, Sop> resultMap = new HashMap<>();
        for (Sop sop : sops) {
            resultMap.put(sop.getSopId(),sop);
        }
        return resultMap;
    }




    private JSONObject getSopListCriteria(List<Sop> content) throws Exception {
        // 遍历结果过滤字段、组合返回查询参数
        JSONObject sopItem,  equipModel;
        JSONObject criteria = new JSONObject();

        if (content != null) {

            HashMap<String, String> equipModelMap = collectEquipModeFromSops(content);
            JSONArray equipModels = new JSONArray();
            for (String equipModelId : equipModelMap.keySet()) {
                equipModel = new JSONObject();
                equipModel.put("equip_model_id", equipModelId);
                equipModel.put("equip_model_name", equipModelMap.get(equipModelId));
                equipModels.add(equipModel);
            }

            HashMap<String, Object> fitObjMap = collectFitObjsFromSops(content);
            JSONArray fitObjs = new JSONArray();
            for (String fitObjId : fitObjMap.keySet()) {
                fitObjs.add(fitObjMap.get(fitObjId));
            }

            HashMap<String, Object> OrderTypeMap = collectOrderTypeFromSops(content);
            JSONArray orderTypes = new JSONArray();
            for (String OrderTypeId : OrderTypeMap.keySet()) {
                orderTypes.add(OrderTypeMap.get(OrderTypeId));
            }

            JSONArray labels = new JSONArray();
            for (Sop sop : content) {
                List<String> labelsItem = sop.getLabels();
                if(CollectionUtils.isNotEmpty(labelsItem)) {
                    for (String s : labelsItem) {
                        if (!labels.contains(s)) {
                            labels.add(s);
                        }
                    }
                }
            }

            //HashMap<String, Object> signMap = getBathchMergeRecordBySign(content);
           // JSONArray signs = new JSONArray();
//			signs.addAll(signMap.values());
           // for (String signCode : signMap.keySet()) {
           //     signs.add(signMap.get(signCode));
            //}
            criteria.put("equip_models", equipModels);
            criteria.put("labels", labels);
            criteria.put("order_type", orderTypes);
            criteria.put("fit_objs", fitObjs);
           // criteria.put("signs", signs);
        }

        return criteria;
    }


    private Map<String, Map<String,String>> getBathchMergeRecordBySign(List<SopInfoVo> sops) {
        Map<String, Map<String,String>> signMap = new HashMap<>();
        if(CollectionUtils.isNotEmpty(sops)) {
            for (SopInfoVo sop : sops) {
                Map<String, String> sign = sop.getSign();
                if (sign != null) {
                    String signCode = sign.get("code");
                    if (!signMap.containsKey(signCode)) {
                        Map<String, String> signTemp = new HashMap<>();
                        signTemp.put("code", signCode);
                        signTemp.put("name", sign.get("name"));
                        signMap.put(signCode, signTemp);
                    }
                }
            }
        }
        return signMap;
    }

    private HashMap<String, Object> collectFitObjsFromSops(List<Sop> sops) {
        HashMap<String, Object> map = new HashMap<>();
        if(CollectionUtils.isNotEmpty(sops)) {
            for (Sop sop : sops) {

                List<Sop.FitObjsBean> fitObjs = sop.getFitObjs();
                if(CollectionUtils.isNotEmpty(fitObjs)) {
                    for (Sop.FitObjsBean fitObj : fitObjs) {
                        Map<String, String> fitObjTemp = new HashMap<>();
                        fitObjTemp.put("obj_id", fitObj.getObj_id());
                        fitObjTemp.put("obj_name", fitObj.getObj_name());
                        map.put(fitObj.getObj_id(), fitObjTemp);
                    }
                }
            }
        }
        return map;
    }
    private HashMap<String, Object> collectOrderTypeFromSops(List<Sop> sops) throws Exception {
        HashMap<String, Object > map = new HashMap<>();
        if(CollectionUtils.isNotEmpty(sops)) {
            for (Sop sop : sops) {
                List<String> orderTypes = sop.getOrderType();
                if(CollectionUtils.isNotEmpty(orderTypes)) {
                    for (String orderTypeId : orderTypes) {
                        Map<String,String> orderTypeMap = new HashMap<>();
                        orderTypeMap.put("code", orderTypeId);
                        orderTypeMap.put("name", orderTypeCache.getOrderTypeNameById(orderTypeId));
                        map.put(orderTypeId, orderTypeMap);
                    }
                }
            }
        }
        return map;
    }


    @Override
    public JSONObject queryEditingSopList(QueryHistoryListVo queryHistoryListVo) throws Exception {
        String sop_name = queryHistoryListVo.getSop_name();

        //根据sop_status  valid  projectid  查询
        QueryWrapper<SopEdit> query = new QueryWrapper<>();
        query.eq("sop_status", SopConst.SOP_STATUS_DRAFT)
                .eq("project_id",queryHistoryListVo.getProjectId())
                .like("sop_name",sop_name)
                .orderByDesc(DBConst.TABLE_FIELD_UPDATE_TIME);

        List<SopEdit> list = sopEditService.list(query);
        List<SopDTO> sopDTOS = new ArrayList<>();
        //转化为业务数据
        for (SopEdit sopEdit : list) {
            SopDTO sopDTO = new SopDTO();
            BeanUtils.copyProperties(sopEdit,sopDTO);
            sopDTOS.add(sopDTO);
        }

        return this.filterRemind(sopDTOS, false);
    }

    @Override
    public JSONObject queryDestroyedSopList(QueryHistoryListVo queryHistoryListVo) throws Exception {

        String sop_name = queryHistoryListVo.getSop_name();

        //根据sop_status  valid  projectid  查询
        QueryWrapper<Sop> query = new QueryWrapper<>();
        query.eq("sop_status", SopConst.SOP_STATUS_DESTROYED)
                .eq("project_id",queryHistoryListVo.getProjectId())
                .like("sop_name",sop_name)
                .orderByDesc(DBConst.TABLE_FIELD_UPDATE_TIME);

        List<Sop> list = sopService.list(query);
        return  this.filterRemind((List)list, true);
    }

    /**
     *
     * Description: 条件过滤
     * @param sops
     * @param destoryFlag
     * @return
     * @throws Exception JSONObject
     * @author yuecaipu
     * @since 2018年5月2日: 下午2:11:59
     * Update By yuecaipu 2018年5月2日: 下午2:11:59
     */
    public JSONObject filterRemind(List<SopDTO> sops, boolean destoryFlag) throws Exception {
        JSONObject sopJson = new JSONObject();
        JSONArray editSopArray = new JSONArray();
        for (SopDTO sop : sops) {
            JSONArray scopes = new JSONArray();

            List<String> orderTypes = sop.getOrderType();

            for (String orderType : orderTypes) {
                String dictName ="tset";//= woFlowPlanService.getNameById(orderType);
                //String dictName = DictionaryCache.getNameByTypeCode(DBConst.DICT_WORK_ORDER_TYPE, tempArray.getString(j), projectId);
                if(dictName != null)
                    scopes.add(dictName);
            }

            List<Sop.FitObjsBean> fitObjs = sop.getFitObjs();
            for (Sop.FitObjsBean fitObj : fitObjs) {
                scopes.add(fitObj.getObj_id());
            }

            JSONObject param = new JSONObject();
            param.put("equip_model_ids", sop.getEquipModelIds());

            String resultStr = equipModelService.queryBathchMergeRecordByequipModelIds(param.toJSONString(), "equip_model_ids", "equip_model_id");
            JSONObject resultObj = JSONObject.parseObject(resultStr);
            JSONArray resultContent = resultObj.getJSONArray(DBConst.Result.CONTENT);
            if (Result.SUCCESS.equals(resultObj.getString(Result.RESULT)) && resultContent.size()>0) {
                for (int j = 0; j < resultContent.size(); j++) {
                    JSONObject content = resultContent.getJSONObject(j);
                    scopes.add(content.getString("equip_model_name"));
                }
            }
            List<String> labels = sop.getLabels();
            if(labels != null) {
                scopes.addAll(labels);
            }

            JSONObject editSop = new JSONObject();
            editSop.put("sop_id", sop.getProjectId());
            editSop.put("sop_name", sop.getSopName());
            editSop.put("version", sop.getVersion());
            if(!destoryFlag) {
                editSop.put("sop_status", sop.getSopStatus());
            }
            editSop.put("scopes", scopes);
            editSop.put("update_time", DateUtil.transferDateFormat(sop.getUpdateTime(), CommonConst.date_format_save, CommonConst.date_format_show));
            editSop.put("operator",sop.getOperator());
            editSopArray.add(editSop);
        }
        sopJson.put("Content", editSopArray);
        sopJson.put("Count", editSopArray.size());
        return sopJson;
    }

    @Override
    public String queryDestroyedSopList(JSONObject jsonObject) throws Exception {
        return null;
    }


    /**
     *
     * Description: 获取到sop中所有设备型号
     * @param sops  集合中的对象中包含equip_model_ids
     * @return HashMap<String,String>    key:equip_model_id ; value:equip_model_name
     * @throws Exception
     * @author yuecaipu
     * @since 2018年7月12日: 上午11:11:17
     * Update By yuecaipu 2018年7月12日: 上午11:11:17
     */
    private HashMap<String, String> collectEquipModeFromSops(List<Sop> sops) throws Exception {

        HashMap<String, String> equipModelMap= new HashMap<>();
        if(CollectionUtils.isNotEmpty(sops)) {
            Set<String> equipModelIds = new HashSet<>();
            for (Sop sop : sops) {
                List<String> equipModelId = sop.getEquipModelIds();
                if(CollectionUtils.isNotEmpty(equipModelIds)) {
                    equipModelIds.addAll(equipModelId);
                }
            }
            if(CollectionUtils.isNotEmpty(equipModelIds)) {
                JSONObject param = new JSONObject();
                param.put("equip_model_ids", equipModelIds);
                String equipModelStrs = equipModelService.queryBathchMergeRecordByequipModelIds(param.toJSONString(),
                        "equip_model_ids", "equip_model_id");
                JSONObject equipModelObjs = JSONObject.parseObject(equipModelStrs);
                JSONArray equipModels = equipModelObjs.getJSONArray(DBConst.Result.CONTENT);
                for (int j = 0, length = equipModels.size(); j < length; j++) {
                    JSONObject equipModel = equipModels.getJSONObject(j);
                    equipModelMap.put(equipModel.getString("equip_model_id"), equipModel.getString("equip_model_name"));
                }
            }
        }
        return equipModelMap;
    }



    /**
     *
     * Description: 补全sop详情信息
     * @param sops
     * @param projectId
     * @param queryType  1-查询发布  2-查询可复制引用
     * @author yuecaipu
     * @throws Exception
     * @since 2018年7月12日: 下午12:23:55
     * Update By yuecaipu 2018年7月12日: 下午12:23:55
     */
    private List<SopInfoVo> additionalSopDetail(List<Sop> sops, String projectId, String queryType) throws Exception {
        List<SopInfoVo> sopInfoVos = new ArrayList<>();
        if(CollectionUtils.isNotEmpty(sops)) {

            HashMap<String, String> equipModelMap = collectEquipModeFromSops(sops);
            for (Sop sop : sops) {
                SopInfoVo sopInfoVo = new SopInfoVo();
                org.springframework.beans.BeanUtils.copyProperties(sop,sopInfoVo);
                if ("2".equals(queryType)) {
                    sopInfoVo.setStep_count(sop.getSteps().size());
                }

                /* if ("1".equals(queryType)) {
                    sop.put("sign", getSopSign(sop, projectId));
                    sop.put("status",projectId.equals(sop.getString("project_id")) ? sop.getString("publish_status") : "0");
                    JSONArray scopes = new JSONArray();
                    JSONArray fitObjs = sop.getJSONArray("fit_objs");
                    for (int j = 0, len = fitObjs.size(); j < len; j++) {
                        scopes.add(fitObjs.getJSONObject(j).getString("obj_name"));
                    }

                    JSONArray equipModelIds = sop.getJSONArray("equip_model_ids");
                    if (equipModelIds != null && equipModelIds.size() > 0) {
                        for (Object equipModelId : equipModelIds) {
                            scopes.add(equipModelMap.get((String) equipModelId));
                        }
                    }
                    JSONArray orderTypes = sop.getJSONArray("order_type");
                    String orderTypeName ;
                    for (Object orderTypeId : orderTypes) {
                        orderTypeName = orderTypeCache.getOrderTypeNameById((String)orderTypeId);
                        scopes.add(orderTypeName);
                    }
                    sop.put("scopes", scopes);
                }*/

                if ("1".equals(queryType)) {
                    sopInfoVo.setSign(getSopSign(sop,projectId));
                }

                sopInfoVos.add(sopInfoVo);
            }
        }
        return sopInfoVos;
    }

    /**
     *
     * @param sop
     * @param projectId
     * @return   code  string
     *           name string
     * @throws Exception
     */
    public Map<String, String> getSopSign(Sop sop, String projectId) throws Exception{
        Map<String,String> sign = null;
        if (CommonConst.group_sop_id.equals(projectId)) {
            String publishStatus = sop.getPublishStatus();
            if (SopConst.PUBLISH_STATUS_REVISE.equals(publishStatus)) {
                sign.put("code", "revise");
                sign.put("name", "待修订");
            } else if (SopConst.PUBLISH_STATUS_REVISEING.equals(publishStatus)) {
                sign.put("code", "revising");
                sign.put("name", "编辑中");
            }
        }else {
            if(SopConst.SOP_TYPE_GROUP.equals(sop.getSopType())){//sop_type:1 集团sop  2项目专用sop
                sign.put("code", "public");
                sign.put("name", "集团");
            }else{
                // String sopProjectId = sop.getString("project_id");
                //  if(projectId.equals(sopProjectId)){
                String publishStatus = sop.getPublishStatus();
                if (SopConst.PUBLISH_STATUS_REVISE.equals(publishStatus)) {
                    sign.put("code", "revise");
                    sign.put("name", "待修订");
                } else if (SopConst.PUBLISH_STATUS_REVISEING.equals(publishStatus)) {
                    sign.put("code", "revising");
                    sign.put("name", "编辑中");
                }
                //}
				/*else{
                sign = new JSONObject();
                sign.put("code", projectId);
                sign.put("name", projectCache.getProjectNameById(projectId));
            }*/
            }
        }
        return sign;
    }



    /**
     *
     * Description: 根据signCodes筛选sop（交集）
     * @param signCodes
     * @param sops
     * @return JSONArray
     * @author yuecaipu
     * @since 2018年7月10日: 下午12:09:41
     * Update By yuecaipu 2018年7月10日: 下午12:09:41
     */
    private List<Sop> filterSopBySignCodes(List<String> signCodes, List<Sop> sops, String projectId) {
        List<Sop> resultSops = new ArrayList<>();
        if(CollectionUtils.isEmpty(signCodes)) {
            return sops;
        }

        if (signCodes.contains("public")) {
            for (Sop object : sops) {
                if (SopConst.SOP_TYPE_GROUP.equals(object.getSopType())) {
                    resultSops.add(object);
                }
            }
        }

        if (signCodes.contains("revise")) {
            for (Sop object : sops) {
                if (projectId.equals(object.getProjectId()) && SopConst.PUBLISH_STATUS_REVISE.equals(object.getPublishStatus())) {
                    resultSops.add(object);
                }
            }
        }

        if (signCodes.contains("revising")) {
            for (Sop object : sops) {
                if (projectId.equals(object.getProjectId()) && SopConst.PUBLISH_STATUS_REVISEING.equals(object.getPublishStatus())) {
                    resultSops.add(object);
                }
            }
        }

        return resultSops;
    }




    public boolean isContainsClassObj(Sop sop) throws Exception {

        List<SopStepsBean> steps = sop.getSteps();
        for (SopStepsBean step : steps) {
            Boolean from_sop = step.isFrom_sop();
            //非引用
            if(!from_sop) {
                List<SopStepsBean.StepContentBean> step_content = step.getStep_content();
                for (SopStepsBean.StepContentBean stepContentBean : step_content) {
                    Boolean from_sop1 = stepContentBean.isFrom_sop();
                    //非引用
                    if(!from_sop1) {
                        List<SopStepsBean.StepContentBean.ContentObjsBean> content_objs = stepContentBean.getContent_objs();
                        for (SopStepsBean.StepContentBean.ContentObjsBean content_obj : content_objs) {
                            String obj_type = content_obj.getObj_type();
                            if (obj_type.contains("_")) {
                                return true;
                            }
                        }
                    }else {
                        //如果是引用的haul
                        String sop_id = stepContentBean.getSop_id();
                        Sop sop1 = new Sop();
                        Sop query = sop1.selectById(sop_id);
                        if(query != null) {
                            return isContainsClassObj(query);
                        }

                    }
                }


            }else {
                //如果是引用的haul
                String sop_id = step.getSop_id();
                Sop sop1 = new Sop();
                Sop query = sop1.selectById(sop_id);
                if(query != null) {
                    return isContainsClassObj(query);
                }

            }
        }
        return false;

    }

    private void removeRelSop( List<Sop>  sops, String projectId, String sopId) {
        Set<String> relSopIds = querySopRel(projectId, sopId);
        relSopIds.add(sopId);
        Iterator<Sop> iterator = sops.iterator();
        while (iterator.hasNext()) {
            Sop next = iterator.next();
            String nodeId = next.getSopId();
            if (relSopIds.contains(nodeId)) {
                iterator.remove();
            }
        }
    }

    private Set<String> querySopRel(String projectId,String sopId) {
        //key：被引用SOP的主键id  value:引用该SOP的SOP主键id
        Map<String, Set<String>> relSopId2sopId = new HashMap<>();

        QueryWrapper<SopSopRel> query = new QueryWrapper();
        query.eq("'",projectId).eq("",sopId).eq("valid",true);

        List<SopSopRel> list = sopSopRelService.list(query);
        for (SopSopRel sopSopRel : list) {
            String rel_sop_id = sopSopRel.getRelSopId();
            String sop_id = sopSopRel.getSopId();
            Set<String> sopIds = relSopId2sopId.getOrDefault(rel_sop_id, new HashSet<String>());
            sopIds.add(sop_id);
            relSopId2sopId.put(rel_sop_id, sopIds);
        }
        return getRelSopRecursively(relSopId2sopId,sopId);
    }

    private Set<String> getRelSopRecursively(Map<String, Set<String>> relSopId2sopId, String sopId) {
        Set<String> relSopIds = new HashSet<>();
        if (CollectionUtils.isEmpty(relSopId2sopId.get(sopId))) {
            return relSopIds;
        }
        Set<String> set = relSopId2sopId.get(sopId);
        relSopIds.addAll(set);
        for (String string : set) {
            relSopIds.addAll(getRelSopRecursively(relSopId2sopId, string));
        }
        return relSopIds;
    }




    private List<Sop> queryPublishedSopsByProjectIdAndUpdatetime(QuerySopListForSelVo sopListForSelVo) throws Exception {
        String sop_name = sopListForSelVo.getSop_name();
        /*
        //如果项目id 不是集团特有的id的话进入
		if (!CommonConst.group_sop_id.equals(requestJson.getString("project_id"))) {
			//如果sign_codes 数时空的话 或者sign_codes 码中包含public 的话
			if (StringUtil.isEmptyList(requestJson, "sign_codes") || requestJson.getJSONArray("sign_codes").contains("public")) {
				//设置为group_sop_id
				projectIds.add(CommonConst.group_sop_id);
			}
		}*/

        QueryWrapper<Sop> query = new QueryWrapper();
        query.eq("project_id",sopListForSelVo.getProjectId())
                .eq("valid",true)
                .eq("sop_status", SopConst.SOP_STATUS_PUBLISH)
                .ge("update_time",sopListForSelVo.getUpdate_time_from())
                .le("update_time",sopListForSelVo.getUpdate_time_to())
                //.ne("publish_status",SopConst.PUBLISH_STATUS_REVISE)
                .like("sop_name",sop_name).orderByDesc("update_time");



        return  sopService.list(query);
    }






    @Override
    public boolean verifySopBeforePublish(VerifySopNameVo verifySopNameVo) throws Exception {
        //查询sop 表中是否存在CommonConst.group_sop_id  sop_name  sopId 的数据
        QueryWrapper<Sop> sopQueryWrapper = new QueryWrapper<>();
        sopQueryWrapper.eq("project_id",CommonConst.group_sop_id).eq("sop_name",verifySopNameVo.getSop_name()).ne("sop_id",verifySopNameVo.getSop_id());
        List<Sop> hashSop = sopService.list(sopQueryWrapper);
        //存在名称
        if(CollectionUtils.isNotEmpty(hashSop)) {
            return false;
        }
        QueryWrapper<SopEdit> sopEditQueryWrapper = new QueryWrapper<>();
        sopEditQueryWrapper.eq("project_id",CommonConst.group_sop_id).eq("sop_name",verifySopNameVo.getSop_name()).ne("sop_id",verifySopNameVo.getSop_id());
        List<SopEdit> hasSopEdit = sopEditService.list(sopEditQueryWrapper);
        //存在名称
        if(CollectionUtils.isNotEmpty(hasSopEdit)) {
            return false;
        }
        return true;
    }



    @Override
    public List<RefSopInfo> queryReferencedSopListById(QuerySopReferenceVo querySopReferenceVo) throws Exception {
        //根据id  和版本进行数据的查询
        QueryWrapper<Sop> sopQueryWrapper = new QueryWrapper<>();
        sopQueryWrapper.eq("sop_id",querySopReferenceVo.getSop_id()).eq("version",querySopReferenceVo.getVersion());
        Sop one = sopService.getOne(sopQueryWrapper);
        // 1通过sop_sop_rel关系表查询到所有引用当前sop的sopID

        QueryWrapper<SopSopRel> sopSopRelQueryWrapper = new QueryWrapper<>();
        sopSopRelQueryWrapper.in("rel_sop_id",one.getSopId()).eq("valid",true);
        List<SopSopRel> sopSopRels = sopSopRelService.list(sopSopRelQueryWrapper);
        Set<String> ids = new HashSet<>();
        for (SopSopRel sopSopRel : sopSopRels) {
            ids.add(sopSopRel.getRelSopId());
        }
        // 2根据sopID 和有效状态 批量查询sop：name、status、update_time、version、stauts_explain

        QueryWrapper<Sop> sopQueryWrapper2 = new QueryWrapper<>();
        sopQueryWrapper2.in("sop_id",ids).eq("valid",true);
        List<Sop> list = sopService.list(sopQueryWrapper2);

        List<RefSopInfo> refSopInfos = new ArrayList<>();
        if(CollectionUtils.isNotEmpty(list)) {
            for (Sop sop : list) {
                RefSopInfo refSopInfo = new RefSopInfo();
                refSopInfo.setRef_sop_id(sop.getSopId());
                refSopInfo.setRef_sop_name(sop.getSopName());
                refSopInfo.setVersion(sop.getVersion());
                refSopInfo.setSop_type(sop.getSopType());
                refSopInfo.setStauts_explain(getSopStatus(sop));
                /*
                 ********************************
                 * ********************************
                 * ********************************
                 * ********************************
                 * ********************************
                 * ********************************
                 * if (!CommonConst.group_sop_id.equals(sopItem.getString("project_id"))) {
                        	sop.put("project_id", sopItem.getString("project_id"));
                        	sop.put("project_name", projectCache.getProjectNameById(sopItem.getString("project_id")));
						}
                 */

                if(sop.getOperator() != null) {
                    refSopInfo.setOperator(JSONObject.toJSONString(sop.getOperator()));
                }
                refSopInfo.setUpdate_time(DateUtil.transferDateFormat(sop.getUpdateTime(), CommonConst.date_format_save, CommonConst.date_format_show));
                refSopInfos.add(refSopInfo);
            }
        }

        //sop 为历史版本的sop
        /*
        ****************************************************************
        * ********************************
        * ********************************
        * ********************************
        * ********************************
        * ****************************************************************
        * ********************************
        * ********************************
        * ********************************
        * ********************************
        *
        *
        *
         */

        return refSopInfos;
    }


    /**
     * 转换sop状态为汉字
     * @param sop
     * @return
     */
    public String getSopStatus(Sop sop){
        String status = sop.getSopStatus();
        if(SopConst.SOP_STATUS_PUBLISH.equals(status)){
            String publishStatus = sop.getPublishStatus();
            status = "已发布";
            if(SopConst.PUBLISH_STATUS_REVISE.equals(publishStatus)){
                status = "已发布、待修订";
            }else if(SopConst.PUBLISH_STATUS_REVISEING.equals(publishStatus)){
                status = "已发布、编辑中";
            }
        }else{
            status = "已作废";
        }
        return status;
    }
}
