package com.persagy.old.cache;

import com.alibaba.fastjson.JSONObject;
import com.persagy.old.common.RedisConstant;
import com.persagy.old.common.StringUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;

@Component
public class ComponentRelationCache {

    private static final Logger log = LoggerFactory.getLogger(ComponentRelationCache.class);
    @Resource(name="saasRedisTemplate")
    private RedisTemplate<String, Object> saasRedisTemplate;

    /**
     * @desc 根据  原始组件编码    查询单个对应关系记录
     * @param baseCmptCode
     * @return
     */
    public JSONObject getComponentRelationByBaseCmptCode(String baseCmptCode) {

        String jsonStr = (String) saasRedisTemplate.opsForHash().get(RedisConstant.ALL_COMPONENT_RELATION_KEY, baseCmptCode);
        if (StringUtil.isNotNull(jsonStr)) {
            return JSONObject.parseObject(jsonStr);
        }
        log.error("ComponentRelationCache:未获得缓存:"+baseCmptCode);
        return new JSONObject();
    }

    /**
     * 根据原始组件编码判断是否有app对应显示组件
     * @param baseCmptCode	原始组件
     * @return
     */
    public boolean hasAppCmpt(String baseCmptCode){
        String app_cmpt_code = getAppComponentCodeByBaseCmptCode(baseCmptCode);
        return StringUtil.isNotNull(app_cmpt_code);
    }

    /**
     * @desc 根据 原始组件编码	查询app的组件code
     * @param baseCmptCode
     * @return
     */
    public String getAppComponentCodeByBaseCmptCode(String baseCmptCode){
        JSONObject componentRelation = getComponentRelationByBaseCmptCode(baseCmptCode);
        return componentRelation.getString("app_cmpt_code");
    }


}
