package com.persagy.old.common;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;

import java.text.SimpleDateFormat;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

public class JSONUtil {
    /**
     * 转换更新参数
     * @param jsonObject    用户接口参数,除了过滤项，其它项默认为新增项
     * @return              数据平台接口参数
     */
    public static JSONObject getAddParamJson(JSONObject jsonObject){
        JSONObject paramJson = new JSONObject();
        paramJson.put("insertObject", jsonObject);
        return paramJson;
    }

    /**
     * 转换批量插入参数-bathchAddParam为JSONObect数组
     * @param batchAddParam
     * @param idName
     * @param idTag
     * @param param
     * @param marjors
     * @return
     */
    public static JSONObject getBatchAddParamJson(JSONArray batchAddParam, String idName, String idTag, JSONObject param, String... marjors){
        JSONArray array = new JSONArray();
        JSONObject batchItem;
        for(int i=0; i<batchAddParam.size(); i++){
            batchItem = batchAddParam.getJSONObject(i);
            batchItem.put(idName, idTag + ToolsUtil.getRecordId(""));
            for(String marjor : marjors){
                batchItem.put(marjor, JSON.parse(JSON.toJSONString(param.get(marjor))));
            }
            array.add(batchItem);
        }
        JSONObject paramJson = new JSONObject();
        paramJson.put("insertObjects", array);
        return paramJson;
    }

    /**
     * 转换批量插入参数-bathch
     * @param batchAddParam   --需要批量插入的数组
     * @param batchName		  --需要插入的字段名
     * @param idName		  --主键字段名称
     * @param idTag           --主键前缀
     * @param param           --携带其它字段参数的对象   
     * @param marjors         --其它字段参数（key）
     * @return
     */
    public static JSONObject getBatchAddParamJson(JSONArray batchAddParam, String batchName, String idName, String idTag, JSONObject param, String... marjors){
        JSONArray array = new JSONArray();
        JSONObject item;
        for(int i=0; i<batchAddParam.size(); i++){
            item = new JSONObject();
            item.put(batchName, batchAddParam.getString(i));
            item.put(idName, idTag + ToolsUtil.getRecordId(""));
            for(String marjor : marjors){
                item.put(marjor, JSON.parse(JSON.toJSONString(param.get(marjor))));
            }
            array.add(item);
        }
        JSONObject paramJson = new JSONObject();
        paramJson.put("insertObjects", array);
        return paramJson;
    }

    /**
     * 转换主键查询参数
     * @param jsonObject    用户接口数据
     * @param majors         主键
     * @return                数据平台接口参数
     */
    public static JSONObject getKeyWithMajors(JSONObject jsonObject, String... majors){
        JSONObject paramJson = new JSONObject();
        JSONObject keyJson = new JSONObject();
        Object value;
        for(String major : majors){
            value = jsonObject.get(major);
            if(value != null && !"".equals(value.toString())){
                keyJson.put(major, value);
            }
        }
        paramJson.put("Key", keyJson);
        return paramJson;
    }

    public static JSONObject getKeysWithMajors(JSONObject jsonObject, String infoName, String infoKey){
        JSONObject paramJson = new JSONObject();
        JSONArray keys = new JSONArray();
        JSONObject key;
        JSONArray keyArray = jsonObject.getJSONArray(infoName);
        if(keyArray != null){
            for(int i=0; i<keyArray.size(); i++){
                key = new JSONObject();
                key.put(infoKey, keyArray.get(i));
                keys.add(key);
            }
        }
        paramJson.put("keys", keys);
        return paramJson;
    }

    public static JSONObject getBatchUpdateParamJson(JSONObject jsonObject, String arrayInfo, String infoName, JSONObject values, String... marjors){
    	JSONObject paramJson = new JSONObject();
    	JSONArray criterias = new JSONArray();
    	
    	JSONArray array = jsonObject.getJSONArray(arrayInfo);
    	if(array == null || array.size() == 0){
    		return null;
    	}
    	for(int i=0; i<array.size(); i++){
    		JSONObject criteria = new JSONObject();
    		criteria.put(infoName, array.get(i));
    		for(String marjor : marjors){
    			criteria.put(marjor, jsonObject.get(marjor));
    		}
    		criterias.add(criteria);
    	}
    	paramJson.put("criterias", criterias);
    	paramJson.put("set", values);
    	
    	return paramJson;
    }
    
    /**
     * 转换更新参数
     * @param jsonObject    用户接口参数,除了过滤项，其它项默认为修改项
     * @param majors        过滤项
     * @return              数据平台接口参数
     */
    public static JSONObject getUpdateParamJson(JSONObject jsonObject, String... majors){
        JSONObject paramJson = new JSONObject();
        JSONObject criteriaJson = new JSONObject();
        Object value;
        for(String major : majors){
            value = jsonObject.get(major);
            if(value != null && !"".equals(value.toString())){
                criteriaJson.put(major, value);
            }
            jsonObject.remove(major);
        }
        paramJson.put("criteria", criteriaJson);
        paramJson.put("set", jsonObject);

        return paramJson;
    }

    /**
     * 转换更新参数
     * @param jsonObject    用户接口参数,除了过滤项，其它项默认为修改项
     * @param conditionName 更新条件名称
     * @param conditionValue 更新条件值
     * @param majors        过滤项
     * @return              数据平台接口参数
     */
    public static JSONObject getUpdateParamJson(String conditionName, String conditionValue, JSONObject jsonObject, String... majors){
        JSONObject paramJson = new JSONObject();
        JSONObject criteriaJson = new JSONObject();
        criteriaJson.put(conditionName, conditionValue);
        Object value;
        for(String major : majors){
            value = jsonObject.get(major);
            if(value != null && !"".equals(value.toString())){
                criteriaJson.put(major, value);
            }
            jsonObject.remove(major);
        }
        paramJson.put("criteria", criteriaJson);
        paramJson.put("set", jsonObject);

        return paramJson;
    }

    /**
     * 获得除去某些项的条件
     * @param jsonObject    用户接口参数
     * @param majors        过滤项
     * @return              数据平台接口参数
     */
    public static JSONObject getCriteriaRemoveMajors(JSONObject jsonObject, String... majors){
        JSONObject paramJson = new JSONObject();
        for(String major : majors){
            jsonObject.remove(major);
        }
        paramJson.put("criteria", jsonObject);
        return paramJson;
    }

    /**
     * 获得指定项的条件
     * @param jsonObject    用户接口参数
     * @param majors        过滤项
     * @return              数据平台接口参数
     */
    public static JSONObject getCriteriaWithMajors(JSONObject jsonObject, String... majors){
        JSONObject paramJson = new JSONObject();
        JSONObject criteria = new JSONObject();
        for(String major : majors){
            criteria.put(major, jsonObject.get(major));
        }
        paramJson.put("criteria", criteria);
        return paramJson;
    }

    /**
     * 获得指定项的条件-用于批量查询
     * @param jsonArray    用户接口参数
     * @param key           参数的code码
     * @param code          用户接口参数code码
     * @return              数据平台接口参数
     */
    public static JSONObject getCriteriasWithMajors(JSONArray jsonArray, String key, String code){
        JSONObject paramJson = new JSONObject();
        JSONArray criterias = new JSONArray();
        JSONObject criteria, json;
        for(int i=0; i<jsonArray.size(); i++){
            json = jsonArray.getJSONObject(i);
            criteria = new JSONObject();
            criteria.put(code, json.get(key));
            criterias.add(criteria);
        }
        paramJson.put("criterias", criterias);
        return paramJson;
    }

    /**
     * 获得指定项的条件-用于批量查询
     * @param jsonObject    用户接口参数
     * @param key           参数的code码
     * @param code          用户接口参数code码
     * @return              数据平台接口参数
     */
    public static JSONObject getCriteriasWithMajors(JSONObject jsonObject, String key, String code){
        JSONObject paramJson = new JSONObject();
        JSONArray criterias = new JSONArray();
        JSONObject criteria = new JSONObject();
        criteria.put(code, jsonObject.get(key));
        criterias.add(criteria);
        paramJson.put("criterias", criterias);
        return paramJson;
    }

    /**
     * 获得批量查询参数
     * @param jsonObject 参数
     * @param arrayInfo 批量数组名
     * @param infoName  查询参数
     * @param marjors   非批量参数
     * @return
     */
    public static JSONObject getCriteriasWithMajors(JSONObject jsonObject, String arrayInfo, String infoName, String... marjors){
        JSONObject paramJson = new JSONObject();
        JSONArray criterias = new JSONArray();
        JSONObject criteria;
        JSONArray infos = jsonObject.getJSONArray(arrayInfo);
        if(infos != null) {
            for (int i=0; i<infos.size(); i++){
                criteria = new JSONObject();
                criteria.put(infoName, infos.get(i));
                for(String marjor : marjors){
                    if(!marjor.equals(infoName)){
                    	Object value = jsonObject.get(marjor);
                    	if(value instanceof JSONObject){
                    		criteria.put(marjor, JSONObject.parseObject((jsonObject.getString(marjor))));
                    	}else if(value instanceof JSONArray){
                    		criteria.put(marjor, JSONArray.parseArray(jsonObject.getString(marjor)));
                    	}else{
                    		criteria.put(marjor, jsonObject.get(marjor));
                    	}
                    }
                }
                criterias.add(criteria);
            }
        }
        paramJson.put("criterias", criterias);
        return paramJson;
    }

    /**
     * 获得数据平台查询条件
     * @param codeName
     * @param jsonObject
     * @param majors
     * @return
     */
    public static JSONObject getDataPlatCriteriaWithMajors(String codeName, JSONObject jsonObject, String... majors){
        JSONObject paramJson = new JSONObject();
        JSONObject criteria = new JSONObject();
        if(codeName != null){
            criteria.put("id", jsonObject.get(codeName));
        }
        for(String major : majors){
            criteria.put(major, jsonObject.get(major));
        }
        paramJson.put("criteria", criteria);
        return paramJson;
    }

    /**
     * filter info
     */
    public static IJSONOperator filter = (jsonObject, filters) -> {
        JSONObject returnJson = new JSONObject();
        for(String filter : filters){
            returnJson.put(filter, jsonObject.get(filter));
        }
        return returnJson;
    };

    /**
     * remove info
     */
    public static IJSONOperator removeInfo = (jsonObject, infos) -> {
        if(jsonObject != null){
            for(String info : infos){
                jsonObject.remove(info);
            }
        }
        return jsonObject;
    };

    /**
     * String to JSONObject/JSONArray
     */
    public static IJSONOperator processStringToJson = (jsonObject, marjors) ->{
        if(jsonObject != null){
            String content;
            for(String marjor : marjors){
                content = jsonObject.getString(marjor);
                if(content != null){
                    if(content.startsWith("{"))
                        jsonObject.put(marjor, JSONObject.parseObject(content));
                    else if(content.startsWith("["))
                        jsonObject.put(marjor, JSONObject.parseArray(content));
                }
            }
        }
        return jsonObject;
    };

    /**
     * tranfer infoValue of date type
     */
    public static IJSONOperator tranferTimeFormat = (jsonObject, marjors) ->{
        SimpleDateFormat fsdf = new SimpleDateFormat(CommonConst.date_format_save);
        SimpleDateFormat tsdf = new SimpleDateFormat(CommonConst.date_format_show);
        for(String marjor : marjors){
            if(jsonObject.containsKey(marjor))
                jsonObject.put(marjor, tsdf.format(fsdf.parse(jsonObject.getString(marjor))));
        }
        return jsonObject;
    };

    /**
     * process result : tranfer info of date type、 filter info、 process string to json
     * @param queryResult query result
     * @param tranferTimeFormat operation of tranfer info of date type
     * @param processStringToJson operation of process string to json
     * @param filter operation of filter info
     * @param dataInfo parameter of tranfer info of date type
     * @param jsonInfo parameter of process string to json
     * @param filters parameter of filter info
     * @return
     * @throws Exception
     */
    public static JSONObject processQueryResult(String queryResult, IJSONOperator tranferTimeFormat, IJSONOperator processStringToJson, IJSONOperator filter, String dataInfo, String[] jsonInfo, String... filters) throws Exception{
        JSONObject queryResultJson = JSONObject.parseObject(queryResult);
        if("failure".equals(queryResultJson.getString("Result")) || queryResultJson.getIntValue("Count") < 1)
            return queryResultJson;
        JSONArray queryResultArray = queryResultJson.getJSONArray("Content");
        JSONArray returnArray = new JSONArray();
        JSONObject item;
        for(int i=0; i<queryResultArray.size(); i++){
            item = queryResultArray.getJSONObject(i);
            if(tranferTimeFormat != null)
                item = tranferTimeFormat.operation(item, dataInfo);
            if(processStringToJson != null)
                item = processStringToJson.operation(item, jsonInfo);
            if(filter != null)
                item = filter.operation(item, filters);
            returnArray.add(item);
        }
        queryResultJson.put("Content", returnArray);
        queryResultJson.put("Count", returnArray.size());
        return queryResultJson;
    }

    /**
     * get first record from query result
     * @param queryResult query result
     * @param tranferTimeFormat operation of tranfer info of date type
     * @param processStringToJson operation of process string to json
     * @param filter operation of filter info
     * @param dataInfo parameter of tranfer info of date type
     * @param jsonInfo parameter of process string to json
     * @param filters parameter of filter info
     * @return
     * @throws Exception
     */
    public static JSONObject getFirstRecordfromResult(String queryResult, IJSONOperator tranferTimeFormat, IJSONOperator processStringToJson, IJSONOperator filter, String[] dataInfo, String[] jsonInfo, String... filters) throws Exception{
        JSONObject queryResultJson = JSONObject.parseObject(queryResult);
        if("failure".equals(queryResultJson.getString("Result")) || queryResultJson.getIntValue("Count") < 1)
            return queryResultJson;
        JSONArray queryResultArray = queryResultJson.getJSONArray("Content");
        JSONObject item = queryResultArray.getJSONObject(0);
        if(tranferTimeFormat != null)
            item = tranferTimeFormat.operation(item, dataInfo);
        if(processStringToJson != null)
            item = processStringToJson.operation(item, jsonInfo);
        if(filter != null)
            item = filter.operation(item, filters);
        queryResultJson.put("Item", item);
        queryResultJson.remove("Content");
        queryResultJson.remove("Count");
        return queryResultJson;
    }

    /**
     *
     * 功能描述：处理查询返回结果，将结果中的内容项转化为JsonString
     * @param queryResult
     * @param majors
     * @return
     * @创建者 wanghailong
     * @邮箱 wanghailong@persagy.com 
     * @修改描述
     */
    public static String prossesResultToJsonString(String queryResult, String... majors){
        if(queryResult.contains("Result") && queryResult.contains("Content")){
            JSONObject resultJson = JSONObject.parseObject(queryResult);
            JSONArray array = (JSONArray) resultJson.get("Content");
            for(int i = 0 ; i < array.size() ; i++){
                JSONObject dataJson = array.getJSONObject(i);
                for(String major : majors){
                    String dataString = dataJson.getString(major);
                    if(!StringUtil.isNull(dataString)){
                        if(dataString.startsWith("{")){
                            dataJson.put(major, JSONObject.parseObject(dataJson.getString(major)));
                        }else if(dataString.startsWith("[")){
                            dataJson.put(major, JSONArray.parseArray(dataJson.getString(major)));
                        }
                    }
                }
                array.set(i, dataJson);
            }
            resultJson.put("Content", array);
            queryResult = resultJson.toString();
        }
        return queryResult;
    }

    /**
     *
     * 功能描述：处理插入参数，将参数中的内容项转化为JsonString
     * @param jsonObject
     * @param majors
     * @return
     * @创建者 wanghailong
     * @邮箱 wanghailong@persagy.com
     * @修改描述
     */
    public static JSONObject prossesParamToJsonString(JSONObject jsonObject, String... majors){
        for(String major : majors){
            if(jsonObject.containsKey(major)){
                jsonObject.put(major, JSON.toJSONString(jsonObject.get(major)));
            }
        }
        return jsonObject;
    }

    /**
     * 功能描述：处理查询返回结果，转换结果中Date类型的数据格式
     * @param queryResult
     * @param fromDateFormat
     * @param toDateFormat
     * @param majors
     * @return
     * @throws Exception
     */
    public static String prossesResultToDateString(String queryResult, String fromDateFormat, String toDateFormat , String... majors) throws Exception{
        if(queryResult.contains("Result") && queryResult.contains("Content")){
            JSONObject resultJson = JSONObject.parseObject(queryResult);
            JSONArray array = (JSONArray) resultJson.get("Content");
            SimpleDateFormat fsdf = new SimpleDateFormat(fromDateFormat);
            SimpleDateFormat tsdf = new SimpleDateFormat(toDateFormat);
            for(int i = 0 ; i < array.size() ; i++){
                JSONObject dataJson = array.getJSONObject(i);
                for(String major : majors){
                    String dataString = dataJson.getString(major);
                    if(!StringUtil.isNull(dataString)){
                        dataJson.put(major, tsdf.format(fsdf.parse(dataString)));
                    }
                }
                array.set(i, dataJson);
            }
            resultJson.put("Content", array);
            queryResult = resultJson.toString();
        }
        return queryResult;
    }

    /**
     *
     * 功能描述：返回查询结果中的第一条记录
     * @param queryResult
     * @return
     * @创建者 wanghailong
     * @邮箱 wanghailong@persagy.com 
     * @修改描述
     */
    public static String getFirstRecordfromResult(String queryResult){
        if(queryResult.contains("Result") && queryResult.contains("Content")){
            JSONObject resultJson = JSONObject.parseObject(queryResult);
            JSONArray array = (JSONArray) resultJson.get("Content");
            if(array.size() > 0 && null != array.get(0)) {
                resultJson.put("Item", JSONObject.parseObject(array.get(0).toString()));
                resultJson.remove("Content");
                resultJson.remove("Count");
            }

            queryResult = resultJson.toString();
        }
        return queryResult;
    }

    /**
     *
     * 功能描述：返回查询结果中的第一条记录,Json格式
     * @param queryResult
     * @return
     * @创建者 wanghailong
     * @邮箱 wanghailong@persagy.com 
     * @修改描述
     */
    public static JSONObject getFirstRecordfromResultWithJson(String queryResult){
        JSONObject firstJson = null;
        if(queryResult.contains("Result") && queryResult.contains("Content")){
            JSONObject resultJson = JSONObject.parseObject(queryResult);
            JSONArray array = (JSONArray) resultJson.get("Content");
            if(array.size() > 0 && null != array.get(0)) {
                firstJson = JSONObject.parseObject(array.get(0).toString());
            }
        }
        return firstJson;
    }

    /**
     * 排序
     * @param queryResult
     * @param field  排序的字段
     * @param order      -1:倒序  1:正序
     * @return
     */
    public static JSONObject sortByField(String queryResult, String field, int order){
        JSONObject resultJson = JSONObject.parseObject(queryResult);
        JSONArray contentArray = resultJson.getJSONArray("Content");
        if(contentArray != null){
            Collections.sort(contentArray, new Comparator<Object>(){
                public int compare(Object str1, Object str2){
                    JSONObject obj1 = JSONObject.parseObject(str1.toString());
                    JSONObject obj2 = JSONObject.parseObject(str2.toString());
                    if(obj1.getLongValue(field) > obj2.getLongValue(field)){
                        return order;
                    }
                    if(obj1.getLongValue(field) == obj2.getLongValue(field)){
                        return 0;
                    }
                    return -1 * order;
                }
            });
        }
        resultJson.put("Content", contentArray);
        return resultJson;
    }

    /**
     * 排序
     * @param array
     * @param field  排序的字段
     * @param order      -1:倒序  1:正序
     * @return
     */
    public static JSONArray sortByStringField(JSONArray array, String field, int order){
    	if(array != null){
    		Collections.sort(array, new Comparator<Object>(){
    			public int compare(Object str1, Object str2){
    				JSONObject obj1 = JSONObject.parseObject(str1.toString());
    				JSONObject obj2 = JSONObject.parseObject(str2.toString());
    				String order1 = obj1.getString(field);
    				if(order1 == null) {
    					order1 = "";
    				}
    				
    				String order2 = obj2.getString(field);
    				if(order2 == null) {
    					order2 = "";
    				}
    				
    				
    				if(order1.compareTo(order2) > -1){
    					return order;
    				}
    				return -1 * order;
    			}
    		});
    	}
    	return array;
    }
    
    /**
     * 排序
     * @param array
     * @param field  排序的字段
     * @param order      -1:倒序  1:正序
     * @return
     */
    public static JSONArray sortByField(JSONArray array, String field, int order){
        if(array != null){
            Collections.sort(array, new Comparator<Object>(){
                public int compare(Object str1, Object str2){
                    JSONObject obj1 = JSONObject.parseObject(str1.toString());
                    JSONObject obj2 = JSONObject.parseObject(str2.toString());
                    if(obj1.getLongValue(field) > obj2.getLongValue(field)){
                        return order;
                    }
                    if(obj1.getLongValue(field) == obj2.getLongValue(field)){
                        return 0;
                    }
                    return -1 * order;
                }
            });
        }
        return array;
    }

    /**
     * 将List转为JSONArray
     * @param list
     * @return
     */
    public static <T> JSONArray parseJSONArray(List<T> list){
        JSONArray array = new JSONArray();
        for(T item : list){
            if(!array.contains(item)){
                array.add(item);
            }
        }
        return array;
    }

    /**
     * 去重JSONArray
     * @param array
     * @return
     */
    public static JSONArray distinctArray(JSONArray array){
        JSONArray distinctedArray = new JSONArray();
        Object item;
        if(array != null) {
            for (int i = 0; i < array.size(); i++) {
                item = array.get(i);
                if (!distinctedArray.contains(item)) {
                    distinctedArray.add(item);
                }
            }
        }
        return distinctedArray;
    }

    
    /**
     * 
     * Description: 在源JSONObject对象中获取目标key的值，组成新的JSONObject并返回
     * @param sourceObj   源string对象
     * @param keys 返回对象的key
     * @return JSONObject 
     * @author yuecaipu
     * @since 2018年7月11日: 下午1:04:10
     * Update By yuecaipu 2018年7月11日: 下午1:04:10
     */
    public static JSONObject getJsonObjectWithKeys(String sourceObj, String... keys) {
    	JSONObject source = JSONObject.parseObject(sourceObj);
    	JSONObject resultObj = getJsonObjectWithKeys(source, keys);
		return resultObj;
	}

    /**
     * 
     * Description: 在源JSONObject对象中获取目标key的值，组成新的JSONObject并返回
     * @param sourceObj 源JSONObject对象
     * @param keys 返回对象的key
     * @return JSONObject 
     * @author yuecaipu
     * @since 2018年7月11日: 下午1:07:49
     * Update By yuecaipu 2018年7月11日: 下午1:07:49
     */
    public static JSONObject getJsonObjectWithKeys(JSONObject sourceObj, String... keys) {
		JSONObject resultObj = new JSONObject();
		for (String key : keys) {
			if (sourceObj.containsKey(key)) {
				resultObj.put(key, sourceObj.get(key));
			}
		}
		return resultObj;
	}
}
