package com.persagy.old.common;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.persagy.old.dao.DBConst;

import java.util.HashMap;

;

public class SopUtil{
	/**
	 * 没有填的必填项放到lackFields集合中
	 * @param sop
	 * @return
	 */
	public static JSONArray validRequiredFields(JSONObject sop){
		JSONArray lackFields = new JSONArray();
		//验证user_id project_id sop_name steps 是否为空 如果为空添加到lackFields 数组中去
		StringUtil.isNull(lackFields, sop, "user_id", "project_id", "sop_name", "steps");
//		StringUtil.isNull(lackFields, sop, "user_id", "project_id", "sop_name", "domains", "steps");
		
		JSONObject fitObj, step, stepContent, contentObj, confirmResult, infoPoint, custom;
		JSONArray fitObjs, steps, stepContents, contentObjs, confirmResults, infoPoints, customs;
		//适用对象
		fitObjs = sop.getJSONArray("fit_objs");
		fitObjs = fitObjs == null ? new JSONArray() : fitObjs;
		for(int i=0; i<fitObjs.size(); i++){
			fitObj = fitObjs.getJSONObject(i);
			//适用对象的obj_id obj_name obj_type 为必须传递的字段
			StringUtil.isNull(lackFields, fitObj, "obj_id", "obj_name", "obj_type");
		}
		prefix(lackFields, "fit_objs", "obj_id", "obj_name", "obj_type");
		
		//步骤-大步
		steps = sop.getJSONArray("steps");
		steps = steps == null ? new JSONArray() : steps;
		for(int i=0; i<steps.size(); i++){
			step = steps.getJSONObject(i);
			
			//步骤-小步
			stepContents = step.getJSONArray("step_content");
			stepContents = stepContents == null ? new JSONArray() : stepContents;
			for(int j=0; j<stepContents.size(); j++){
				stepContent = stepContents.getJSONObject(j);
				
				//小步涉及到的对象
				contentObjs = stepContent.getJSONArray("content_objs");
				//涉及的对象的 obj_id obj_name obj_type 不能为空
				contentObjs = contentObjs == null ? new JSONArray() : contentObjs;
				for(int k=0; k<contentObjs.size(); k++){
					contentObj = contentObjs.getJSONObject(k);
					StringUtil.isNull(lackFields, contentObj, "obj_id", "obj_name", "obj_type");
				}
				prefix(lackFields, "content_objs", "obj_id", "obj_name", "obj_type");
				JSONArray contents = stepContent.getJSONArray("contents");
				contents = contents == null ? new JSONArray() : contents;
				for (int l = 0; l < contents.size(); l++) {
					JSONObject content = contents.getJSONObject(l);
					//contents 的 description 不能为空
					StringUtil.isNull(lackFields, content, "description");
					//小步需确认的反馈信息
					confirmResults = content.getJSONArray("confirm_result");
					confirmResults = confirmResults == null ? new JSONArray() : confirmResults;
					for(int k=0; k<confirmResults.size(); k++){
						confirmResult = confirmResults.getJSONObject(k);
						
						//反馈信息中的信息点
						infoPoints = confirmResult.getJSONArray("info_points");
						infoPoints = infoPoints == null ? new JSONArray() : infoPoints;
						for(int m=0; m<infoPoints.size(); m++){
							infoPoint = infoPoints.getJSONObject(m);
							//信息点 的id code name 不能为空
							StringUtil.isNull(lackFields, infoPoint, "id", "code", "name");
						}
						prefix(lackFields, "info_points", "id", "code", "name");
						
						//反馈信息中的自定义信息

						customs = confirmResult.getJSONArray("customs");
						customs = customs == null ? new JSONArray() : customs;
						for(int m=0; m<customs.size(); m++){
							custom = customs.getJSONObject(m);
							//自定义信息的 name type 不能为空
							StringUtil.isNull(lackFields, custom, "name", "type");
						}
						prefix(lackFields, "customs", "name", "type");
						//confirm_result 的 obj_id obj_name obj_type 不能为空
						StringUtil.isNull(lackFields, confirmResult, "obj_id", "obj_name", "obj_type");
					}
					prefix(lackFields, "confirm_result", "obj_id", "obj_name", "obj_type");
				}
				//step_content 的from_sop 不能为空
				StringUtil.isNull(lackFields, stepContent, "from_sop");
			}
			prefix(lackFields, "step_content", "from_sop");
			//steps 的from_sop 不能为空
			StringUtil.isNull(lackFields, step, "from_sop");
		}
		prefix(lackFields, "steps", "from_sop");
		
		return lackFields;
	}
	
	private static void prefix(JSONArray lackFields, String prefix, String... params){
		lackFields = lackFields == null ? new JSONArray() : lackFields;
		for(String param : params){
			if(lackFields.contains(param)){
				if(!lackFields.contains(prefix + "." + param))
					lackFields.add(prefix + "." + param);
				lackFields.remove(param);
			}
		}
	}
	
	/**
	 * 获取自定义信息点code
	 * @param code			主信息点code，异常信息点必填
	 * @param objType		对象类型，自定义信息点必填
	 * @param endWith		结束标志，异常信息点时有用，默认“status”
	 * @return
	 */
	public static String getCtmInfoPointCode(String code, String objType, String endWith){
		String ctmCode;
		if(StringUtil.isNull(code)){
			ctmCode = DBConst.ID_TAG_CTM_INFO_POINT + "_" + objType  + "_" + ToolsUtil.getUuid();
		}else{
			if(StringUtil.isNull(endWith)){
				endWith = "status";
			}
			if(!code.startsWith(DBConst.ID_TAG_CTM_INFO_POINT)){
				code = DBConst.ID_TAG_CTM_INFO_POINT + "_" + code;
			}
			ctmCode = code + "_" + endWith;
		}
		return ctmCode;
	}

	/**
	 * 
	 * Description:  合并两个sop的查询结果
	 * @param resultStrA
	 * @param resultStrB
	 * @return String
	 * @author yuecaipu
	 * @since 2018年4月27日: 下午3:20:40
	 * Update By yuecaipu 2018年4月27日: 下午3:20:40
	 */
	public static String integrateSopResult(String resultStrA, String resultStrB) {
		JSONObject resultJsonA = JSONObject.parseObject(resultStrA);
		JSONObject resultJsonB = JSONObject.parseObject(resultStrB);
		JSONArray resultContentA = resultJsonA.getJSONArray("Content");
		JSONArray resultContentB = resultJsonB.getJSONArray("Content");
		if (resultContentA==null || resultContentA.size()==0) {
			return resultStrB;
		}
		if (resultContentB==null || resultContentB.size()==0) {
			return resultStrA;
		}
		resultContentA.addAll(resultContentB);
		resultJsonA.put("Content",resultContentA);
		return resultJsonA.toJSONString();
	}
	
	 /**
	 * Description: 获取sop所有关系对象（适用对象和步骤中的对象并集）
	 * @param sop
	 * @param sopObjs
	 * @return JSONArray
	 * @author yuecaipu
	 * @since 2018年5月28日: 下午5:21:21
	 * Update By yuecaipu 2018年5月28日: 下午5:21:21
	 */
	 
	public static JSONArray getSopRelObjs(JSONObject sop) {
		JSONArray sopObjs = sop.getJSONArray("fit_objs");
		HashMap<String, Object> sopRelObjsMap = new HashMap<String, Object>();
		for (int i = 0, length = sopObjs.size(); i < length; i++) {
			sopRelObjsMap.put(sopObjs.getJSONObject(i).getString("obj_id"), sopObjs.getJSONObject(i));
		}        
		JSONArray steps = sop.getJSONArray("steps");
		JSONObject step,stepContent,contentObj;
		JSONArray stepContents,contentObjs;
		String objId;
		for (int i = 0, stepsLength = steps.size(); i < stepsLength; i++) {
			step = steps.getJSONObject(i);
			if (!step.getBooleanValue("from_sop")) {
				stepContents = step.getJSONArray("step_content");
				if (stepContents != null && stepContents.size() > 0) {
					for (int j = 0, stepContentsLength = stepContents.size(); j < stepContentsLength; j++) {
						stepContent = stepContents.getJSONObject(j);
						// 工作内容中涉及的对象
						contentObjs = stepContent.getJSONArray("content_objs");
						if (contentObjs != null && contentObjs.size() > 0) {
							for (int k = 0, contentObjsLength = contentObjs.size(); k < contentObjsLength; k++) {
								contentObj = contentObjs.getJSONObject(k);
								objId = contentObj.getString("obj_id");
								if (!sopRelObjsMap.containsKey(objId)) {
									sopRelObjsMap.put(objId, contentObj);
								}
							}
						}
					}
				}
			}
		}
		//sop中 适用对象和步骤中的对象
		JSONArray sopRelObjs = new JSONArray();
		for (Object obj : sopRelObjsMap.values()) {
			sopRelObjs.add(obj);
		}
		return sopRelObjs;
	}
}
