/**
 * @包名称 com.sagacloud.service
 * @文件名 BaseService.java
 * @创建者 wanghailong
 * @邮箱 wanghailong@persagy.com  
 * @修改描述 
 */

package com.persagy.old.service;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.persagy.old.common.*;
import com.persagy.old.dao.DBConst;
import com.persagy.old.dao.DBConst.Result;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.http.NameValuePair;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.client.methods.HttpUriRequest;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.message.BasicNameValuePair;
import org.apache.http.util.EntityUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.util.*;

/**
 * 功能描述：
 * 
 * @类型名称 BaseService
 * @创建者 wanghailong
 * @邮箱 wanghailong@persagy.com
 * @修改描述
 * 
 * update by yuecaipu 2018/5/2
 */
//被@RefreshScope修饰的Bean都是延迟加载的，只有在第一次访问时才会被初始化，刷新Bean也是同理，下次访问时会创建一个信息的对象
//@RefreshScope //这里不能省略  不然不能手动触发
@Service("baseService")
public class BaseService {
	@Value("${data.platform.basePath}")
	public String dataPlatBasePath;


	/**
	 * 数据平台使用设备标签筛选
    */
    @Value("${obj.filter.by.tag}")
    public boolean tagFlag;
    /**
     * 数据平台设备标签（后缀）
     */
    @Value("${equip.tag}")
    public String equipTag;

	private static final Logger log = LoggerFactory.getLogger(BaseService.class);

	/**
	 * 功能描述：http请求，Json类型参数
	 * 
	 * @param url
	 * @param jsonStr
	 * @return
	 * @throws Exception
	 * @创建者 wanghailong
	 * @邮箱 wanghailong@persagy.com 修改描述
	 */
	public String httpPostRequest(String url, String jsonStr) throws Exception {
		log.info("请求数据平台post："+url);
		
		if(StringUtil.isNull(jsonStr)) {
			jsonStr = new JSONObject().toJSONString();
		}
		/**
		 * 统一处理：调用非运维系统时，user_id统一使用systemId
		 */
//		JSONObject jsonParam = JSONObject.parseObject(jsonStr);
//		jsonParam.put("user_id", CommonConst.SYSTEM_ID);
//		jsonStr = jsonParam.toJSONString();
		
		/***********统一处理：调用非运维系统时，user_id统一使用systemId  结束**************************/
		
		
		log.info(jsonStr);

		/*---- 数据平台不接收get方式传参 需要将get请求的参数projectId转为post updateTime:20180504 10:53 ----*/
//		dataPlatBasePath = dataPlatBasePath == null ? initDataPlatBasePath() : dataPlatBasePath;
//		if (url.contains("projectId") && url.contains(dataPlatBasePath)) {
//			JSONObject jsonObject = JSONObject.parseObject(jsonStr);
//			int index = url.indexOf("projectId");
//			String projectId = url.substring(index + "projectId=".length(), index + "projectId".length() + 13);
//			jsonObject.put("projectId", projectId);
//			jsonStr = jsonObject.toString();
//		}
		/*----------- end -------------*/

		String respContent = null;
		CloseableHttpClient httpclient = HttpClients.createDefault();
		HttpPost httpPost = new HttpPost(url);
		httpPost.addHeader("Content-Type", "application/json;charset=utf-8");
		httpPost.setEntity(new StringEntity(jsonStr, "utf-8"));
		// 执行请求
		respContent = executeHttpRequest(httpclient, httpPost);
//		log.info(respContent);
		return respContent;
	}

	
	/**
	 * Description: post方式访问数据平台接口，项目id放到post参数体内，不需要项目密码
	 * @param url 请求数据平台url
	 * @param jsonStr 请求参数jsonString
	 * @return
	 * @throws Exception String
	 * @author cuixubin
	 * @since 2018年5月3日: 下午8:11:04
	 * Update By cuixubin 2018年5月3日: 下午8:11:04
	 */
	public String httpPostRequestDataPlatform(String url, String jsonStr) throws Exception {

        /**
         * 数据平台设备标签筛选 特殊处理
         **/
        if (tagFlag) {
            if (url.contains("equipment/complex_query")) {
                JSONArray businessTag = new JSONArray();
                businessTag.add(equipTag);
                JSONObject jsonObject = JSONObject.parseObject(jsonStr);
                jsonObject.put("businessTag", businessTag);
                jsonStr = jsonObject.toJSONString();
            } else if (url.contains("object/search")) {
                JSONObject jsonObject = JSONObject.parseObject(jsonStr);
                JSONArray category = jsonObject.getJSONArray("category");
                if (CollectionUtils.isNotEmpty(category)) {
                    JSONArray businessTag = new JSONArray();
                    for (int i = 0; i < category.size(); i++) {
                        if ("EqObj".equals(category.getString(i))) {  //目前只支持设备对象可以打标签
                            businessTag.add(category.getString(i) + "_" + equipTag);
                        }
                    }
                    jsonObject.put("businessTag", businessTag);
                    jsonStr = jsonObject.toJSONString();
                }
            } else if (url.contains("object/subset_query") || url.contains("object/query_by_info")) {
                JSONObject jsonObject = JSONObject.parseObject(jsonStr);
                JSONObject criteria = (JSONObject) jsonObject.getOrDefault("criteria", new JSONObject());
                JSONArray type = (JSONArray) criteria.getOrDefault("type", new JSONArray());
                if (CollectionUtils.isNotEmpty(type)) {
                    JSONArray businessTag = new JSONArray();
                    for (int i = 0; i < type.size(); i++) {
                        if ("Eq".equals(type.getString(i))) {  //目前只支持设备对象可以打标签
                            businessTag.add(type.getString(i) + "_" + equipTag);
                        }
                    }
                    jsonObject.put("businessTag", businessTag);
                    jsonStr = jsonObject.toJSONString();
                }
            }
        }

        log.info("请求数据平台post："+url);
        log.info(jsonStr);

        // 项目id放到post体
//		if(url.contains("projectId")){
//			JSONObject jsonObject = JSONObject.parseObject(jsonStr);
//			int index = url.indexOf("projectId");
//			String projectId = url.substring(index+"projectId=".length(),index+"projectId".length()+13);
//			jsonObject.put("projectId",projectId);
//			jsonStr = jsonObject.toString();
//		}

		String respContent = null;
		CloseableHttpClient httpclient = HttpClients.createDefault();
		HttpPost httpPost = new HttpPost(url);
		httpPost.addHeader("Content-Type", "application/json;charset=utf-8");
		httpPost.setEntity(new StringEntity(jsonStr, "utf-8"));
		// 执行请求
		respContent = executeHttpRequest(httpclient, httpPost);
		if(null != respContent && respContent.contains("failure")) {
			log.error(respContent);
		}
		return respContent;
	}
	
	

	
	
	
	/**
	 * 
	 * 功能描述：：http请求，非Json类型参数，参数直接是'&'间隔的
	 * 
	 * @param url
	 * @param params
	 * @return
	 * @throws Exception
	 * @创建者 wanghailong
	 * @邮箱 wanghailong@persagy.com 修改描述
	 */
	@Deprecated
	public String httpPostWithListParams(String url, Map<String, String> params) throws Exception {
		String respContent = null;
		CloseableHttpClient httpclient = HttpClients.createDefault();
		HttpPost httpPost = new HttpPost(url);
		httpPost.addHeader("Content-Type", "application/json;charset=utf-8");
		// 设置请求的的参数
		List<NameValuePair> nvps = new ArrayList<NameValuePair>();
		// 参数 jsonArray形式
		Set<String> keySet = params.keySet();
		for (String key : keySet) {
			nvps.add(new BasicNameValuePair(key, params.get(key)));
		}
		httpPost.setEntity(new UrlEncodedFormEntity(nvps, "utf-8"));
		// 执行请求
		respContent = executeHttpRequest(httpclient, httpPost);
//		log.info(respContent);
		return respContent;
	}


	
	/**
	 * 功能描述：http请求，Json类型参数
	 *
	 * @param url
	 * @param jsonStr
	 * @return
	 * @throws Exception
	 * @创建者 wanghailong
	 * @邮箱 wanghailong@persagy.com 修改描述
	 */
	public String httpPostRequestToPersagy(String url, String jsonStr) throws Exception {
		log.info("请求数据平台post："+url);
		log.info(jsonStr);
		String respContent = null;
		CloseableHttpClient httpclient = HttpClients.createDefault();
		HttpPost httpPost = new HttpPost(url);
		httpPost.addHeader("Content-Type", "application/x-www-form-urlencoded");
		httpPost.setEntity(new StringEntity("jsonString=" + jsonStr, "utf-8"));
		// 执行请求
		respContent = executeHttpRequest(httpclient, httpPost);
//		log.info(respContent);
		return respContent;
	}

	
	
	
	
	private Map<String, String> URLRequest(String URL) {
		Map<String, String> mapRequest = new HashMap<String, String>();
		String[] arrSplit = null;
		String strUrlParam = TruncateUrlPage(URL);
		if (strUrlParam == null) {
			return mapRequest;
		}
		//每个键值为一组 www.2cto.com
		arrSplit = strUrlParam.split("[&]");
		for (String strSplit : arrSplit) {
			String[] arrSplitEqual = null;
			arrSplitEqual = strSplit.split("[=]");

			//解析出键值
			if (arrSplitEqual.length > 1) {
				//正确解析
				mapRequest.put(arrSplitEqual[0], arrSplitEqual[1]);
			} else {
				if (arrSplitEqual[0] != "") {
					//只有参数没有值，不加入
					mapRequest.put(arrSplitEqual[0], "");
				}
			}
		}
		mapRequest.remove("secret");
		return mapRequest;
	}

	private static String TruncateUrlPage(String strURL) {
		if (StringUtils.isNotEmpty(strURL)) {
			String[] arrSplit = strURL.trim().split("[?]");
			if (arrSplit.length > 1) {
				return arrSplit[1];
			}
		}
		return null;
	}


	/**
	 * 执行http请求
	 * @param httpClient
	 * @param httpRequest
	 * @return
	 */
	private static String executeHttpRequest(CloseableHttpClient httpClient, HttpUriRequest httpRequest){
		CloseableHttpResponse response;
		String respContent = null;
		try {
			response = httpClient.execute(httpRequest);
			respContent = EntityUtils.toString(response.getEntity(), "utf-8");
		} catch (Exception e) {
		}finally {
			try {
				httpClient.close();
			} catch (IOException e) {
				log.error("HttpClient closed failed!");
			}
			
			if(respContent == null || respContent.startsWith("<")){
				respContent = ToolsUtil.errorJsonMsg("请求异常" + respContent);
			}
		}
		return respContent;
	}

	  /**
	    *
	    * 功能描述：：获取数据平台请求路径
	    * @param requestPath 请求路径，DataRequestPathUtil中的属性值
	    * @return
	    * @创建者 wanghailong
	    * @邮箱 wanghailong@persagy.com
	    * 修改描述
	    */
		public String getDataPlatformPath(String requestPath) {
			String requestUrl = null;
			if (null == requestPath || "".equals(requestPath))
				return requestUrl;
			else
				requestUrl = requestPath;
			requestUrl = requestUrl.replace("BASEPATH", dataPlatBasePath);
//			requestUrl = requestUrl.replace("SECRET", dataPlatDefaultSecret);
			return requestUrl;
		}
		/**
		 *
		 * 功能描述：获取数据平台-数据字典请求路径
		 * @param requestPath
		 * @param param 数据字典编码
		 * @return
		 * @创建者 wanghailong
		 * @邮箱 wanghailong@persagy.com
		 * 修改描述
		 */
		public String getDataPlatDictPath(String requestPath , String param) {
			String requestUrl = null;
			if (null == requestPath || "".equals(requestPath))
				return requestUrl;
			else
				requestUrl = requestPath;
			requestUrl = requestUrl.replace("BASEPATH", dataPlatBasePath);
			if(null != param && !"".equals(param)){
				requestUrl = requestUrl.replace("PARAM", param);
			}
			return requestUrl;
		}
		
	/**
	 *
	 * 功能描述：获取数据平台请求路径
	 * @param requestPath  请求路径，DataRequestPathUtil中的属性值
	 * @param projectId 项目Id，项目相关请求，path中可以不带该参数
	 * @param secret 秘钥，项目相关请求可以使用默认秘钥，其它请求需要projectId和secret匹配
	 * @return
	 * @创建者 wanghailong
	 * @邮箱 wanghailong@persagy.com 修改描述
	 */
//	public String getDataPlatformPath(String requestPath, String projectId, String secret) {
//		String requestUrl = null;
//		if (null == requestPath || "".equals(requestPath))
//			return requestUrl;
//		else
//			requestUrl = requestPath;
//		requestUrl = requestUrl.replace("BASEPATH", dataPlatBasePath);
//		if (null != projectId && !"".equals(projectId)) {
//			requestUrl = requestUrl.replace("PROJECTID", projectId);
//		}
////		if (null != secret && !"".equals(secret)) {
////			requestUrl = requestUrl.replace("SECRET", secret);
////		} else {
////			requestUrl = requestUrl.replace("SECRET", dataPlatDefaultSecret);
////		}
//		return requestUrl;
//	}
	
	/**
	 * 获取创建自定义信息点参数中的objType
	 * @param objId
	 * @param objType
	 * @param objTypeMap 暂存
	 * @return
	 */
	public String getObjTypeForCtm(String objId, String objType, Map<String, String> objTypeMap) throws Exception {
		objTypeMap = objTypeMap == null ? Maps.newHashMap() : objTypeMap;
		String ctmObjType = objTypeMap.get(objId);
		if(!StringUtil.isNull(ctmObjType)){
		}else if(objType.equals(CommonConst.data_type_build)){
			ctmObjType = "building";
		}else if(objType.equals(CommonConst.data_type_floor)){
			ctmObjType = "floor";
		}else if(objType.equals(CommonConst.data_type_space)){
			ctmObjType = "space";
		}else if(objType.equals(CommonConst.data_type_equip)){
			String project_id = StringUtil.transferId(objId, CommonConst.tag_project, dataPlatBasePath);
			String params = "{\"projectId\":\""+ project_id +"\", \"criterias\":[{\"id\":\""+objId+"\"}]}";
			String requestUrl = getDataPlatformPath(DataRequestPathUtil.dataPlat_object_batch_query);
//			String resultStr = httpPostRequest(requestUrl, params);
			String resultStr = httpPostRequestDataPlatform(requestUrl, params);
			JSONObject resultJson = JSONObject.parseObject(resultStr);
			JSONArray content = resultJson.getJSONArray(DBConst.Result.CONTENT);
			if(content != null && content.size() != 0){
				JSONObject item = content.getJSONObject(0);
				JSONObject relation = item.getJSONObject("relation");
				relation = relation == null ? new JSONObject() : relation;
				ctmObjType = relation.getString("equipment_category");
			}
		}else if(objType.equals(CommonConst.data_type_equip_class)){
			ctmObjType = objId;
		}else if(objType.equals(CommonConst.data_type_system)){
			String project_id = StringUtil.transferId(objId, CommonConst.tag_project, dataPlatBasePath);
			String params = "{\"projectId\":\""+ project_id +"\", \"criterias\":[{\"id\":\""+objId+"\"}]}";
			String requestUrl = getDataPlatformPath(DataRequestPathUtil.dataPlat_object_batch_query);
//			String resultStr = httpPostRequest(requestUrl, params);
			String resultStr = httpPostRequestDataPlatform(requestUrl, params);
			JSONObject resultJson = JSONObject.parseObject(resultStr);
			JSONArray content = resultJson.getJSONArray(Result.CONTENT);
			if(content != null && content.size() != 0){
				JSONObject item = content.getJSONObject(0);
				JSONObject relation = item.getJSONObject("relation");
				relation = relation == null ? new JSONObject() : relation;
				ctmObjType = relation.getString("equipment_category");
			}
		}else if(objType.equals(CommonConst.data_type_system_class)){
			ctmObjType = objId;
		}
		objTypeMap.put(objId, ctmObjType);
		return ctmObjType;
	}

	/**
	 * 获取信息点的id中的对象类型
	 * @param objType	confirm_result中的对象类型
	 * @return
	 */
	public String getObjTypeForInfoId(String objType) throws Exception {
		String ctmObjType = null;
		if(objType.equals(CommonConst.data_type_build)){
			ctmObjType = "build";
		}else if(objType.equals(CommonConst.data_type_floor)){
			ctmObjType = "floor";
		}else if(objType.equals(CommonConst.data_type_space)){
			ctmObjType = "space";
		}else if(objType.equals(CommonConst.data_type_equip)){
			ctmObjType = "equip";
		}else if(objType.equals(CommonConst.data_type_equip_class)){
			ctmObjType = "equip";
		}else if(objType.equals(CommonConst.data_type_system)){
			ctmObjType = "system";
		}else if(objType.equals(CommonConst.data_type_system_class)){
			ctmObjType = "system";
		}
		return ctmObjType;
	}
	
	/**
	 * 创建自定义信息点，自定义信息点的异常信息点一定要在自定义信息点后面，返回结果的map中不存在infoPointCode的信息点表示创建失败，调用之后用返回结果中的新code替换原来的code
	 * @param objType					对象类型，如果有多种类型的对象，分开多次调用即可
	 * @param firstTag				自定义信息点默认的一级标签
	 * @param unit						单位
	 * @param dataType				数据类型，默认Str
	 * @param customInfos				自定义信息点
	 * @return			<调用方法传入的信息点的code， 实际在数据平台存储的code>
	 */
	public Map<String, String> createCustomInfos(String objType, String firstTag, String unit, String dataType, List<JSONObject> customInfos) throws Exception {
		Map<String, String> resultMap = Maps.newHashMap();
		//设置默认值
		if(StringUtil.isNull(firstTag)){
			firstTag = "反馈参数";
		}
		if(StringUtil.isNull(dataType)){
			dataType = "Str";
		}
		//1.查询objType对象类型的自定义信息点有哪些
		//1.1 查询方案ID
		String schemeId = null;
		JSONObject params = new JSONObject();
		String paramStr = DataRequestProcessUtil.getCriteriaWithMajors(params).toJSONString();
		String requestUrl = getDataPlatformPath(DataRequestPathUtil.dataPlat_custom_infoCode_scheme_query);
//		String resultStr = httpPostRequest(requestUrl, paramStr);
		String resultStr = httpPostRequestDataPlatform(requestUrl, paramStr);
		JSONObject resultJson = JSONObject.parseObject(resultStr);
		if(Result.FAILURE.equals(resultJson.getString(Result.RESULT))){
			return resultMap;
		}
		JSONArray schemes = resultJson.getJSONArray(Result.CONTENT);
		if(schemes == null || schemes.size() == 0){
			return resultMap;
		}
		JSONObject scheme = schemes.getJSONObject(0);
		schemeId = scheme.getString("scheme_id");
		//1.2 已存在的信息点
		params.put("scheme_id", schemeId);
		params.put("obj_type", objType);
		params.put("firstTag", firstTag);
		paramStr = DataRequestProcessUtil.getCriteriaWithMajors(params, "scheme_id", "obj_type", "firstTag").toJSONString();
		requestUrl = getDataPlatformPath(DataRequestPathUtil.dataPlat_custom_infoCode_scheme_info_query);
//		resultStr = httpPostRequest(requestUrl, paramStr);
		resultStr = httpPostRequestDataPlatform(requestUrl, paramStr);
		resultJson = JSONObject.parseObject(resultStr);
		if(Result.FAILURE.equals(resultJson.getString(Result.RESULT))){
			return resultMap;
		}
		//信息点名称和code对照信息
		Map<String, String> infoNameOriginalCode = DataRequestProcessUtil.getMapFromResult(resultJson, "infoPointName", "infoPointCode");

		//2.补充自定义信息点，并记录信息点在数据平台的code
		customInfos = customInfos == null ? Lists.newArrayList() : customInfos;
		JSONObject customInfo;
		String infoPointCode, infoPointName, mainInfoPointCode, originalInfoPointCode, inputMode;
		for(int i=0; i<customInfos.size(); i++){
			customInfo = customInfos.get(i);
			infoPointCode = customInfo.getString("infoPointCode");
			infoPointName = customInfo.getString("infoPointName");
			mainInfoPointCode = customInfo.getString("mainInfoPointCode");
			originalInfoPointCode = infoNameOriginalCode.getOrDefault(infoPointName, infoPointCode);

			if(!StringUtil.isNull(mainInfoPointCode)){
				if(!mainInfoPointCode.equals(resultMap.get(mainInfoPointCode))){
					//替换自定义信息点的异常信息点的code
					originalInfoPointCode = infoPointCode.replace(mainInfoPointCode, resultMap.get(mainInfoPointCode));
				}
			}
			//记录infoPointCode的变化
			resultMap.put(infoPointCode, originalInfoPointCode);

			customInfo.put("firstTag", firstTag);
			customInfo.put("infoPointCode", originalInfoPointCode);
			customInfo.put("dataType", dataType);
			customInfo.put("inputMode", getInfoPointOriginalModeFromAppMode(customInfo.getString("inputMode")));
		}
		//3.创建自定义信息点
		params.clear();
		params.put("scheme_id", schemeId);
		params.put("obj_type", objType);
		params.put("criterias", customInfos);
		requestUrl = getDataPlatformPath(DataRequestPathUtil.dataPlat_custom_infoCode_scheme_info_create);
//		resultStr = httpPostRequest(requestUrl, params.toJSONString());
		resultStr = httpPostRequestDataPlatform(requestUrl, params.toJSONString());
		resultJson = JSONObject.parseObject(resultStr);
		if(Result.FAILURE.equals(resultJson.getString(Result.RESULT))){
			resultMap.clear();
		}
		return resultMap;
	}

	/**
	 * 根据自定义信息点类型转换信息点原始组件
	 * @param appMode
	 * @return
	 */
	public String getInfoPointOriginalModeFromAppMode(String appMode){
		if(StringUtil.isNull(appMode)){
			appMode = "default";
		}
		String infoPointMode = CommonMessage.customTypeInfoPointModeRels.getString(appMode);
		if(StringUtil.isNull(infoPointMode)){
			infoPointMode = CommonMessage.customTypeInfoPointModeRels.getString("default");
		}
		return infoPointMode;
	}
	
}
