package com.persagy.old.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.persagy.old.cache.GraphCache;
import com.persagy.old.cache.ProjectCache;
import com.persagy.old.common.*;
import com.persagy.old.dao.DBCommonMethods;
import com.persagy.old.dao.DBConst;
import com.persagy.old.dao.DBConst.Result;
import com.persagy.old.service.BaseService;
import com.persagy.old.service.ObjectServiceI;
import com.persagy.old.service.WoFlowPlanServiceI;
import org.apache.commons.collections.CollectionUtils;
import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.Map.Entry;

/**
 * 
 * Description: Company: Persagy
 * 
 * @author yuecaipu
 * @version 2.0
 * @since: 2018年7月18日: 上午10:00:00 Update By yuecaipu 2018年8月8日: 上午8:88:88
 */
@Service("woFlowPlanService")
public class WoFlowPlanServiceImpl2 extends BaseService implements WoFlowPlanServiceI {

	private ThreadLocal<JSONObject> skipsCache = new ThreadLocal<>();
	private static final Logger log = Logger.getLogger(WoFlowPlanServiceImpl2.class);
	@Autowired
	public ObjectServiceI objectService;
	@Autowired
	public GraphCache graphCache;
	@Autowired
	public ProjectCache projectCache;

	@Override
	public String getNameById(String orderTypeId) {
		String paramStr, resultStr, orderTypeName = null;
		JSONObject resultObj;
		paramStr = "{\"criteria\":{\"plan_id\":\"" + orderTypeId + "\"}}";
		resultStr = DBCommonMethods.queryRecordByCriteria(DBConst.DATABASE_WORK_ORDER,
				DBConst.TABLE_WO_FLOW_PLAN_PERSAGY, paramStr);
		resultObj = JSONObject.parseObject(resultStr);
		if (Result.SUCCESS.equals(resultObj.getString(Result.RESULT)) && resultObj.getInteger(Result.COUNT) > 0) {
			orderTypeName = resultObj.getJSONArray(Result.CONTENT).getJSONObject(0).getString("plan_name");
		}
		return orderTypeName;
	}

	@Override
	public String autoInstantiateWoMatters(JSONObject jsonObject) throws Exception {
		
		JSONObject item = instantiateObjs(jsonObject);
		
		int count = 0;
		//如果没有实例化的对象，再实例化一次,最多实例化5次
		while (isEnptyInstantiateObj(item) && count<4) {
			count++;
			item = instantiateObjs(jsonObject);			
		}
		// 移除缓存
		spaceRelationMapChache.remove();
		allSpaceChache.remove();
		allFloorChache.remove();
		allShaftChache.remove();
		
		return ToolsUtil.successJsonMsg(item);
	}

    //判断结果中是否有实例化对象
    private boolean isEnptyInstantiateObj(JSONObject item){
    	boolean flag = false;
    	JSONArray published_matters = item.getJSONArray("published_matters");
		int total = 0;
    	A:for (int i = 0; i < published_matters.size(); i++) {
			JSONObject matter = published_matters.getJSONObject(i);
			JSONArray matterSteps = matter.getJSONArray("matter_steps");
			if (CollectionUtils.isEmpty(matterSteps)) {
				break ;
			}
			for (int j = 0; j < matterSteps.size(); j++) {
				JSONObject step = matterSteps.getJSONObject(j);
				if ("5".equals(step.getString("step_type"))) {
					JSONArray step_content = step.getJSONArray("step_content");
					if (CollectionUtils.isEmpty(step_content)) {
						break ;
					}			
					for (int k = 0; k < step_content.size(); k++) {
						JSONObject content = step_content.getJSONObject(k);
						JSONArray confirm_result = content.getJSONArray("confirm_result");
						if (confirm_result != null) {
							total += confirm_result.size();
							if (total > 0) {
								break A;
							}
						}
					}
				}
			}
		}
		if (total == 0) {
			flag = true;
		}
		return flag;
	}	
	 /**
	 * Description: 
	 * @param jsonObject
	 * @return
	 * @throws Exception JSONObject
	 * @author yuecaipu
	 * @since 2018年8月23日: 上午11:45:38
	 * Update By yuecaipu 2018年8月23日: 上午11:45:38
	 */
	 
	private JSONObject instantiateObjs(JSONObject jsonObject) throws Exception {
		JSONArray publishedMatters = jsonObject.getJSONArray("published_matters");
		String projectId = jsonObject.getString("project_id");
		// 1.查询skips放入缓存
		if (!StringUtil.isNull(jsonObject, "plan_id")) {
			JSONObject skips = querySkipsByWOPlanId(jsonObject.getString("plan_id"));

			skipsCache.set(skips);
			updateSkipsByPlanId(jsonObject.getString("plan_id"));
		}
		// 2.实例化Matter
		JSONObject item = new JSONObject();
		JSONArray instantiateMatters = new JSONArray();
		JSONObject instantiateMatter;
		for (int i = 0; i < publishedMatters.size(); i++) {
			instantiateMatter = instantiateMatter(publishedMatters.getJSONObject(i), projectId);
			instantiateMatters.add(instantiateMatter);
		}
		skipsCache.remove();
		item.put("published_matters", instantiateMatters);
		return item;
	}
	
	/**
	 * 更新wo_plan的skips内容
	 */
	private void updateSkipsByPlanId(String planId) {
		JSONObject param = new JSONObject();
		JSONObject skips = skipsCache.get();
		
		Integer frequency = skips.getInteger("frequency");
		frequency = frequency == null ? 0 : frequency;
		Integer frequency1 = frequency + 1;
		JSONObject skips1 = new JSONObject();
		skips1.put("frequency", frequency1);
		param.put("plan_id", planId);
		param.put("skips", skips1.toJSONString());
		String paramStr = JSONUtil.getUpdateParamJson(param, "plan_id").toJSONString();
		String resultStr = DBCommonMethods.updateRecord(DBConst.DATABASE_WORK_ORDER, DBConst.TABLE_WO_PLAN, paramStr,
				false);
		if (resultStr.contains(Result.FAILURE)) {
			log.info("计划执行频率更新失败！！！");
		}
	}

	// 查询工单已执行次数
	private JSONObject querySkipsByWOPlanId(String planId) throws Exception {
		JSONObject param = new JSONObject();
		JSONObject key = new JSONObject();
		JSONArray keys = new JSONArray();
		key.put("plan_id", planId);
		keys.add(key);
		param.put("keys", keys);
		JSONArray returnColumns = new JSONArray();
		returnColumns.add("skips");
		param.put("ReturnColumns", returnColumns);
		String resultStr = DBCommonMethods.getBatchRecordBykeys(DBConst.DATABASE_WORK_ORDER, DBConst.TABLE_WO_PLAN,
				param.toJSONString());
		JSONObject resultJson = JSONObject.parseObject(resultStr);
		JSONObject skips = null;
		if (Result.SUCCESS.equals(resultJson.getString(Result.RESULT))) {
			JSONArray content = resultJson.getJSONArray(Result.CONTENT);
			skips = JSONObject.parseObject(content.getJSONObject(0).getString("skips"));
			skips = skips == null ? new JSONObject() : skips;
			if (!StringUtil.isNull(skips, "resetTime") && new Date()
					.after(DateUtil.parseDate(CommonConst.date_format_save, skips.getString("resetTime")))) {
				skips.clear();
			}
		}
		return skips;
	}

	/**
	 * 实例化matter
	 * 
	 * @param projectId
	 */
	private JSONObject instantiateMatter(JSONObject matter, String projectId) throws Exception {
		JSONObject instantiateMatter = (JSONObject) matter.clone();
		JSONArray matterSteps = matter.getJSONArray("matter_steps");
		if (CollectionUtils.isEmpty(matterSteps)) {
			return instantiateMatter;
		}
		JSONArray instantiateMatterSteps = new JSONArray();
		JSONObject instantiateMatterStep;
		for (int i = 0; i < matterSteps.size(); i++) {
			instantiateMatterStep = instantiateMatterStep(matterSteps.getJSONObject(i), projectId);
			instantiateMatterSteps.add(instantiateMatterStep);
		}
		instantiateMatter.put("matter_steps", instantiateMatterSteps);
		return instantiateMatter;
	}

	/**
	 * 实例化matterStep
	 */
	private JSONObject instantiateMatterStep(JSONObject matterStep, String projectId) throws Exception {
		JSONObject instantiateMatterStep = (JSONObject) matterStep.clone();

		JSONArray stepContents = matterStep.getJSONArray("step_content");

		if (CollectionUtils.isEmpty(stepContents)) {
			return instantiateMatterStep;
		}
		JSONArray instantiateStepContents = instantiateMatterStep.getJSONArray("step_content");

		if ("5".equals(matterStep.getString("step_type"))) {// step_type-步骤类型：1-文字输入、5-工作内容、6-签字, 7-事项拍照 非空
			instantiateStepContents = new JSONArray();
			JSONObject stepContent;
			for (Object object : stepContents) {
				stepContent = (JSONObject) object;
				stepContent = instantiateStepContent(stepContent, projectId);
				instantiateStepContents.add(stepContent);
			}
			instantiateMatterStep.put("step_content", instantiateStepContents);
		}

		return instantiateMatterStep;
	}

	/**
	 * 实例化StepContent
	 */
	private JSONObject instantiateStepContent(JSONObject stepContent, String projectId) throws Exception {
		JSONObject instantiateStepContent = (JSONObject) stepContent.clone();
		JSONArray confirmResults = stepContent.getJSONArray("confirm_result");
		if (CollectionUtils.isEmpty(confirmResults)) {
			return instantiateStepContent;
		}
		JSONArray instantiateConfirmResults = new JSONArray();
		JSONArray instantiateObjs;
		for (Object object : confirmResults) {
			instantiateObjs = instantiateConfirmResult((JSONObject) object, projectId);
			instantiateConfirmResults.addAll(instantiateObjs);
		}
		instantiateStepContent.put("confirm_result", instantiateConfirmResults);
		return instantiateStepContent;
	}

	/**
	 * 实例化对象
	 */
	private JSONArray instantiateConfirmResult(JSONObject confirmResult, String projectId) throws Exception {
		JSONArray instantiateObjs = new JSONArray();
		JSONObject instantiateConfirmResult = (JSONObject) confirmResult.clone();

		// 对象类型
		String objType = confirmResult.getString("obj_type");
		// 是否手动添加对象实例 1-是，0-否 默认0
		String manual_obj_flag = confirmResult.getString("manual_obj_flag");
		manual_obj_flag = manual_obj_flag == null ? "0" : manual_obj_flag;

		if (!objType.endsWith("_class")) {
			instantiateObjs.add(instantiateConfirmResult);
		} else if ("0".equals(manual_obj_flag)) {// 自动实例化

			if ("system_class".equals(objType)) {// 实例化系统类
				JSONArray systemObjs = instantiateSystemClass(confirmResult, projectId);
				instantiateObjs.addAll(systemObjs);
			}

			if ("equip_class".equals(objType)) {// 实例化设备类
				JSONArray equipObjs = instantiateEquipClass(confirmResult, projectId);
				instantiateObjs.addAll(equipObjs);
			}

			if ("space_class".equals(objType)) {// 实例化空间类
				JSONArray spaceObjs = instantiateSpaceClass(confirmResult, projectId);
				instantiateObjs.addAll(spaceObjs);
			}
		}

		return instantiateObjs;
	}

	private ThreadLocal<JSONArray> allSpaceChache = new ThreadLocal<>();
	private ThreadLocal<JSONArray> allFloorChache = new ThreadLocal<>();
	private ThreadLocal<JSONArray> allShaftChache = new ThreadLocal<>();
	private ThreadLocal<JSONArray> allBuildChache = new ThreadLocal<>();

	private JSONArray getAllSpace(String projectId) throws Exception {
		JSONArray allSpace = allSpaceChache.get();
		if (allSpace == null) {
			allSpace = subset_query(projectId, "GeneralZone");
			allSpaceChache.set(allSpace);
		}
		return allSpace;
	}
	private JSONArray getAllBuild(String projectId) throws Exception {
		JSONArray allBuild = allBuildChache.get();
		if (allBuild == null) {
			allBuild = subset_query(projectId, "Bd");
			allBuildChache.set(allBuild);
		}
		return allBuild;
	}
	private JSONArray getAllFloor(String projectId) throws Exception {
		JSONArray allFloor = allFloorChache.get();
		if (allFloor == null) {
			allFloor = subset_query(projectId, "Fl");
			allFloorChache.set(allFloor);
		}
		return allFloor;
	}

	private JSONArray getAllShaft(String projectId) throws Exception {
		JSONArray allShaft = allShaftChache.get();
		if (allShaft == null) {
			allShaft = subset_query(projectId, "Sh");
			allShaftChache.set(allShaft);
		}
		return allShaft;
	}

	// 获取空间范围内的所有空间
	private JSONArray querySpacesByScopeV(JSONArray scopes, String projectId) throws Exception {
		JSONObject scope;
		JSONArray resultArray = null;
		String ScopeType = null;
		if (scopes != null && scopes.size() > 0) {
			ScopeType = scopes.getJSONObject(0).getString("obj_type");
			if (ScopeType != null && "space_class".equals(ScopeType)) {
				JSONObject param = new JSONObject();
				JSONObject criteria = new JSONObject();
				JSONArray infos = new JSONArray();
				criteria.put("type", JSONArray.parseArray("[\"Sp\"]"));
				criteria.put("infos", infos);
				param.put("projectId", projectId);
				param.put("criteria", criteria);
				for (Object object : scopes) {
					scope = (JSONObject) object;
					JSONObject info = new JSONObject();
					info.put("ProjRoomFuncType", scope.getString("obj_id"));
					infos.add(info);
				}
				resultArray = query_by_info(param);

			} else if (ScopeType != null && "floor_identity".equals(ScopeType)) {
				// JSONArray allSpaces = subset_query(projectId, "GeneralZone");
				// JSONArray floors = subset_query(projectId, "Fl");
				JSONArray allSpaces = getAllSpace(projectId);
				JSONArray floors = getAllFloor(projectId);
				Map<String, List<JSONObject>> floor2Spaces = assortSpaceByFloor(allSpaces);
				Map<String, List<String>> floorIdentity2Floors = assortFloorByFloorIdentity(floors);
				List<String> floorIdsForScope = new ArrayList<>();
				for (Object object : scopes) {
					scope = (JSONObject) object;
					List<String> floorIdsForIdentity = floorIdentity2Floors.get(scope.getString("obj_id"));
					if (floorIdsForIdentity != null && floorIdsForIdentity.size() != 0) {
						floorIdsForScope.addAll(floorIdsForIdentity);
					}
				}
				resultArray = new JSONArray();
				for (String floorId : floorIdsForScope) {
					if (floor2Spaces.get(floorId) != null) {
						resultArray.addAll(floor2Spaces.get(floorId));
					}
				}
			} else if (ScopeType != null && "shaft_type".equals(ScopeType)) {
				// TODO
			}
		}else {
			resultArray = getAllSpace(projectId);
		}
		return resultArray;
	}

	// 获取空间范围内的所有空间
	private JSONArray querySpacesByScope(JSONArray scopes, String projectId) throws Exception {
		JSONObject scope;
		JSONArray resultArray = new JSONArray();
		JSONArray allSpaces = getAllSpace(projectId);

		if (scopes != null && scopes.size() > 0) {
			String ScopeType = scopes.getJSONObject(0).getString("obj_type");
			if (ScopeType != null && "space_class".equals(ScopeType)) {
				Map<String, List<JSONObject>> roomFuncType2spaces = assortSpaceByRoomFuncType(allSpaces);
				for (Object object : scopes) {
					scope = (JSONObject) object;
					if (roomFuncType2spaces.get(scope.getString("obj_id")) != null) {
						resultArray.addAll(roomFuncType2spaces.get(scope.getString("obj_id")));
					}
				}
			} else if (ScopeType != null && "floor_identity".equals(ScopeType)) {
				JSONArray floors = getAllFloor(projectId);
				Map<String, List<JSONObject>> floor2spaces = assortSpaceByFloor(allSpaces);
				Map<String, List<String>> floorIdentity2floors = assortFloorByFloorIdentity(floors);
				List<String> floorIdsForScope = new ArrayList<>();
				for (Object object : scopes) {
					scope = (JSONObject) object;
					List<String> floorIdsForIdentity = floorIdentity2floors.get(scope.getString("obj_id"));
					if (floorIdsForIdentity != null && floorIdsForIdentity.size() != 0) {
						floorIdsForScope.addAll(floorIdsForIdentity);
					}
				}
				for (String floorId : floorIdsForScope) {
					if (floor2spaces.get(floorId) != null) {
						resultArray.addAll(floor2spaces.get(floorId));
					}
				}
			} else if (ScopeType != null && "shaft_type".equals(ScopeType)) {
				// TODO
			}
		} else {
			resultArray = allSpaces;
		}

		return resultArray;
	}
	
	//空间按空间类型分类
	private Map<String, List<JSONObject>> assortSpaceByRoomFuncType(JSONArray allSpaces) {
		Map<String, List<JSONObject>> roomFuncType2spaces = new HashMap<>();
		for (Object object : allSpaces) {
			JSONObject space = (JSONObject) object;
			JSONObject infos = space.getJSONObject("infos");
			if (infos != null && !infos.isEmpty()) {
				String roomFuncType = infos.getString("ProjRoomFuncType");
				List<JSONObject> list = roomFuncType2spaces.getOrDefault(roomFuncType, new ArrayList<>());
				list.add(space);
				roomFuncType2spaces.put(roomFuncType, list);
			}
		}
		return roomFuncType2spaces;
	}
	//楼层按编码分类
	private Map<String, List<String>> assortFloorByFloorIdentity(JSONArray floors) {
		Map<String, List<String>> floorIdentity2floors = new HashMap<>();
		for (Object object : floors) {
			JSONObject floor = (JSONObject) object;
			JSONObject infos = floor.getJSONObject("infos");
			if (infos != null && !infos.isEmpty()) {
				String FloorIdentity = infos.getString("FloorIdentity");
				List<String> list = floorIdentity2floors.getOrDefault(FloorIdentity, new ArrayList<>());
				list.add(floor.getString("id"));
				floorIdentity2floors.put(FloorIdentity, list);
			}
		}
		return floorIdentity2floors;
	}
	//空间按楼层分类
	private Map<String, List<JSONObject>> assortSpaceByFloor(JSONArray allSpaces) {
		Map<String, List<JSONObject>> floor2spaces = new HashMap<>();
		for (Object object : allSpaces) {
			JSONObject space = (JSONObject) object;
			JSONObject infos = space.getJSONObject("location");
			if (infos != null && !infos.isEmpty()) {
				String floorId = infos.getString("floor");
				List<JSONObject> list = floor2spaces.getOrDefault(floorId, new ArrayList<>());
				list.add(space);
				floor2spaces.put(floorId, list);
			}
		}
		return floor2spaces;
	}

	/**
	 * 实例化空间类对象 先实例化所有空间类对象，再根据筛选条件获取到所有排序对象的id，
	 */
	private JSONArray instantiateSpaceClass(JSONObject confirmResult, String projectId) throws Exception {
		JSONArray resultArray;
		// 计划每次是否执行全部对象 1-是，0-否 默认1
		String execute_all_obj_flag = confirmResult.getString("execute_all_obj_flag");
		execute_all_obj_flag = execute_all_obj_flag == null ? "1" : execute_all_obj_flag;

		// 1.实例化全部对象
		JSONArray allInstanceObj = instanceAllObj(confirmResult, projectId);
		if (allInstanceObj.isEmpty()) {
			return allInstanceObj;
		}
		// 2.根据空间范围筛选
		if (confirmResult.getJSONArray("scope") != null && confirmResult.getJSONArray("scope").size() != 0) {
			JSONArray scopes = confirmResult.getJSONArray("scope");
			String objType = scopes.getJSONObject(0).getString("obj_type");
			if ("space_class".equals(objType)) {
				boolean flag = false;
				for (Object object : scopes) {
					JSONObject scope = (JSONObject) object;
					if (scope.getString("obj_id").equals(confirmResult.getString("obj_id"))) {
						flag = true;
					}
				}
				if (!flag) {
					return new JSONArray();
				}
			} else {
				allInstanceObj = filterSapceByScope(confirmResult.getJSONArray("scope"), allInstanceObj, projectId);
			}
		}

		if ("1".equals(execute_all_obj_flag)) {
			resultArray = allInstanceObj;
		} else {
			// 对象执行排序 floor-楼层，shaft-竖井，space-空间
			String sequence_id_type = confirmResult.getString("sequence_id_type");
			JSONArray spaceIds = null;
			if ("floor".equals(sequence_id_type)) {
				// 获取所有空间所对应的楼层
				JSONArray floorIds = getFloorIds(confirmResult, projectId);

				// 2.根据楼层id获取所有空间
				spaceIds = getSpaceIdsByFloorIds(floorIds);
			}

			if ("shaft".equals(sequence_id_type)) {
				JSONArray shaftIds = getShaftIds(confirmResult, projectId);
				// 2.根据竖井id获取所有空间
				spaceIds = getSpaceIdsByShaftIds(projectId, shaftIds);
			}

			if ("space".equals(sequence_id_type)) {
				spaceIds = getSpaceIds(allInstanceObj, confirmResult, projectId);

			}
			// 3.取交集
			resultArray = new JSONArray();
			for (int i = 0; i < allInstanceObj.size(); i++) {
				JSONObject obj = allInstanceObj.getJSONObject(i);
				String obj_id = obj.getString("obj_id");
				if (spaceIds.contains(obj_id)) {
					resultArray.add(obj);
				}
			}
		}
		return resultArray;
	}

	/**
	 * 实例化设备类对象
	 * 
	 * @throws Exception
	 */
	private JSONArray instantiateEquipClass(JSONObject confirmResult, String projectId) throws Exception {
		JSONArray resultArray;
		// 计划每次是否执行全部对象 1-是，0-否 默认1
		String execute_all_obj_flag = confirmResult.getString("execute_all_obj_flag");
		execute_all_obj_flag = execute_all_obj_flag == null ? "1" : execute_all_obj_flag;

		// 1.实例化全部设备对象
		JSONArray allInstanceObj = instanceAllObj(confirmResult, projectId);
		if (allInstanceObj.isEmpty()) {
			return allInstanceObj;
		}
		// 2.根据空间范围筛选
		allInstanceObj = filterEquipByScope(confirmResult.getJSONArray("scope"), allInstanceObj, projectId);

		if ("1".equals(execute_all_obj_flag)) {
			resultArray = allInstanceObj;
		} else {
			// 对象执行排序 floor-楼层，shaft-竖井，equip-设备
			String sequence_id_type = confirmResult.getString("sequence_id_type");
			JSONArray spaceIds = null;
			JSONArray equipIds = new JSONArray();
			if ("floor".equals(sequence_id_type)) {
				JSONArray floorIds = getFloorIds(confirmResult, projectId);
				// 2.根据楼层id获取所有空间
				spaceIds = getSpaceIdsByFloorIds(floorIds);
//				spaceIds = filterSpaceId(spaceIds,querySpacesByScope(confirmResult.getJSONArray("scope"), projectId));
				// 3.根据房间id获取房间内所有设备
				equipIds = getEquipIdsBySpaceIds(spaceIds);
			}

			if ("shaft".equals(sequence_id_type)) {
				JSONArray shaftIds = getShaftIds(confirmResult, projectId);
				// 2.根据竖井id获取所有空间
				spaceIds = getSpaceIdsByShaftIds(projectId, shaftIds);
//				spaceIds = filterSpaceId(spaceIds,querySpacesByScope(confirmResult.getJSONArray("scope"), projectId));
				// 3.根据房间id获取房间内所有设备
				equipIds = getEquipIdsBySpaceIds(spaceIds);
			}

			if ("equip".equals(sequence_id_type)) {
				equipIds = getEquipIds(allInstanceObj, confirmResult, projectId);
			}

			// 3.取交集
			resultArray = new JSONArray();
			for (int i = 0; i < allInstanceObj.size(); i++) {
				JSONObject obj = allInstanceObj.getJSONObject(i);
				String obj_id = obj.getString("obj_id");
				if (equipIds.contains(obj_id)) {
					resultArray.add(obj);
				}
			}
		}
		return resultArray;
	}

	private JSONArray filterSpaceId(JSONArray spaceIds, JSONArray allInstanceObj) {
		JSONArray reulst = new JSONArray();
		for (Object object : allInstanceObj) {
			JSONObject space = (JSONObject)object;
			if (spaceIds.contains(space.getString("id"))) {
				reulst.add(space.getString("id"));
			}
		}
		return reulst;
	}

	private JSONArray filterSapceByScope(JSONArray scopes, JSONArray allInstanceSpaceObj, String projectId)
			throws Exception {
		if (scopes == null || scopes.isEmpty()) {
			return allInstanceSpaceObj;
		}
		JSONArray spaceForScope = querySpacesByScope(scopes, projectId);
		JSONArray spaceIdsForScope = new JSONArray();
		JSONObject space;
		for (Object object : spaceForScope) {
			space = (JSONObject) object;
			spaceIdsForScope.add(space.getString("id"));
		}
		JSONArray resultArray = new JSONArray();
		for (Object object : allInstanceSpaceObj) {
			space = (JSONObject) object;
			if (spaceIdsForScope.contains(space.getString("obj_id"))) {
				resultArray.add(space);
			}
		}
		return resultArray;
	}


	private JSONArray getSpaceIdsByShaftIds(String projectId, JSONArray shaftIds) throws Exception {
		// 请求数据平台relation/query接口 获取图关系
		String graphId = graphCache.getGraphIdByProjectIdAndGraph(projectId, "ShaftandSpace");
		if (StringUtil.isNull(graphId)) {
			// 图实例id
			graphId = graphCache.createGraphInstance(projectId,"ShaftandSpace");
		}
		JSONObject param = new JSONObject();
		JSONArray criterias = new JSONArray();
		param.put("projectId", projectId);
		param.put("merge", true);
		param.put("criterias", criterias);
		JSONObject criteria;
		for (int i = 0; i < shaftIds.size(); i++) {
			criteria = new JSONObject();
			criteria.put("graph_id", graphId);
			criteria.put("from_id", shaftIds.getString(i));
			criteria.put("rel_type", "1");
			criterias.add(criteria);
		}
//		String url = getDataPlatformPath(DataRequestPathUtil.dataPlat_relation_relation_instance_query, projectId,
//				null);
		String url = getDataPlatformPath(DataRequestPathUtil.dataPlat_relation_relation_instance_query);

//		String resultStr = httpPostRequest(url, param.toJSONString());
		String resultStr = httpPostRequestDataPlatform(url, param.toJSONString());
		JSONObject resultObj = JSONObject.parseObject(resultStr);

		// 遍历结果 提取空间id
		JSONArray result = new JSONArray();
		if (Result.SUCCESS.equals(resultObj.getString(Result.RESULT))) {
			JSONArray resultContent = resultObj.getJSONArray(Result.CONTENT);
			if (resultContent != null) {
				for (int i = 0; i < resultContent.size(); i++) {
					JSONObject content = resultContent.getJSONObject(i);
					String spaceId = content.getString("to_id");
					result.add(spaceId);
				}
			}
		}

		return result;
	}

	private JSONArray getSpaceIdsByFloorIds(JSONArray floorIds) throws Exception {
		if (floorIds.isEmpty()) {
			return new JSONArray();
		}
		// 先获取到项目下的所有空间
		String projectId = "Pj" + floorIds.getString(0).substring(2, 12);
		// JSONArray spaces = subset_query(projectId, "GeneralZone");
		JSONArray spaces = getAllSpace(projectId);
		// 将空间与楼层一一对应 {floorId:[ spaceId1,spaceId2 ]}
		JSONObject floorSpaces = new JSONObject();
		JSONArray spaceIds;
		for (int i = 0; i < spaces.size(); i++) {
			JSONObject space = spaces.getJSONObject(i);
			String spaceId = space.getString("id");
			JSONObject location = space.getJSONObject("location");
			if (location != null) {
				String floor = location.getString("floor");
				if (floor != null) {
					spaceIds = (JSONArray) floorSpaces.getOrDefault(floor, new JSONArray());
					spaceIds.add(spaceId);
					floorSpaces.put(floor, spaceIds);
				}
			}
		}
		// 根据楼层id获取的所属的空间id
		JSONArray resultSpaceids = new JSONArray();
		for (int i = 0; i < floorIds.size(); i++) {
			String floorId = floorIds.getString(i);
			JSONArray spaceids = floorSpaces.getJSONArray(floorId);
			if (spaceids != null) {
				resultSpaceids.addAll(spaceids);
			}
		}

		return resultSpaceids;
	}

	private JSONArray getSpaceIds(JSONArray allInstanceObj, JSONObject confirmResult, String projectId)
			throws Exception {
		if (allInstanceObj != null) {
			JSONObject param = new JSONObject();
			JSONArray criterias = new JSONArray();
			param.put("projectId", projectId);
			param.put("criterias", criterias);
			JSONObject criteria;
			for (int i = 0; i < allInstanceObj.size(); i++) {
				JSONObject obj = allInstanceObj.getJSONObject(i);
				String objId = obj.getString("obj_id");
				criteria = new JSONObject();
				criteria.put("id", objId);
				criterias.add(criteria);
			}
			String url = getDataPlatformPath(DataRequestPathUtil.dataPlat_object_batch_query_post);
//			String resultStr = httpPostRequest(url, param.toJSONString());
			String resultStr = httpPostRequestDataPlatform(url, param.toJSONString());
			JSONObject resultObj = JSONObject.parseObject(resultStr);

			if (Result.SUCCESS.equals(resultObj.getString(Result.RESULT))) {
				JSONArray spaces = resultObj.getJSONArray(Result.CONTENT);
				Map<String, JSONArray> sequenceIdAndSpaceMap = getSequenceIdAndObjMap(spaces);
				// 筛选房间
				JSONArray result = filterBySequence(sequenceIdAndSpaceMap, confirmResult);
				return result;
			}

		}
		return new JSONArray();
	}

	private JSONArray getShaftIds(JSONObject confirmResult, String projectId) throws Exception {
		JSONArray shafts = null;
		// 1.获取项目下所有竖井，并排序
		if (confirmResult.getJSONArray("scope") == null || confirmResult.getJSONArray("scope").isEmpty()) {
			// shafts = subset_query(projectId, "Sh");
			shafts = getAllShaft(projectId);
		} else {

			JSONArray spaceForScope = querySpacesByScope(confirmResult.getJSONArray("scope"), projectId);
			shafts = new JSONArray();
			JSONObject param = new JSONObject();
			JSONArray criterias = new JSONArray();
			JSONObject item;
//			for (Object object : spaceForScope) {
//				JSONObject space = (JSONObject) object;
//				String shaftId = getShaftIdBySpaceId(projectId, space.getString("id"));
//				if (shaftId != null) {
//					item = new JSONObject();
//					item.put("id", shaftId);
//					criterias.add(item);
//				}
//			}
			//根据空间获取竖井id
			JSONArray shaftIds = queryShaftIdsFromSpace(spaceForScope,projectId);
			for (Object object : shaftIds) {
				item = new JSONObject();
				item.put("id", object);
				criterias.add(item);
			}
			param.put("projectId", projectId);
			param.put("criterias", criterias);
			shafts = batch_query(param);

		}

		Map<String, JSONArray> sequenceIdAndShaftMap = getSequenceIdAndObjMap(shafts);
		// 2.筛选竖井
		JSONArray resultShaftIds = filterBySequence(sequenceIdAndShaftMap, confirmResult);
		return resultShaftIds;

	}

	private JSONArray queryShaftIdsFromSpace(JSONArray spaceForScope, String projectId) throws Exception {
		JSONArray spaceIds = new JSONArray();
		JSONArray ShaftIds = new JSONArray();
		for (Object object : spaceForScope) {
			JSONObject space = (JSONObject)object;
			spaceIds.add(space.getString("id"));
		}
		String graphId = graphCache.getGraphIdByProjectIdAndGraph(projectId, "ShaftandSpace");
		if (StringUtil.isNull(graphId)) {
			// 图实例id
            graphId = graphCache.createGraphInstance(projectId,"ShaftandSpace");
		}
		JSONArray relTypes = new JSONArray();
		relTypes.add("1");
		JSONArray ShaftandSpace = relation_query_subset(projectId, graphId, relTypes).getJSONObject(0).getJSONArray("Content");
		for (Object object : ShaftandSpace) {
			JSONObject ralation = (JSONObject)object;
			if (spaceIds.contains(ralation.getString("to_id"))) {
				ShaftIds.add(ralation.getString("from_id"));
			}
		}
		return ShaftIds;
	}

	private String getShaftIdBySpaceId(String project_id, String space_id) throws Exception {
		String graph_id = graphCache.getGraphIdByProjectIdAndGraph(project_id, "ShaftandSpace");
		if (StringUtil.isNull(graph_id)) {
			// 图实例id
            graph_id = graphCache.createGraphInstance(project_id,"ShaftandSpace");
		}
		if (graph_id != null && !"".equals(graph_id)) {
			String secret = projectCache.getProjectSecretById(project_id);
			JSONArray queryRelations = queryRelations(project_id, secret, null, space_id, graph_id, "1");
			if (!queryRelations.isEmpty()) {
				JSONObject shaft = queryRelations.getJSONObject(0);
				String shaftId = shaft.getString("from_id");
				return shaftId;
			}
		}
		return null;
	}

	/**
	 * 功能描述： 获取数据平台的关系实例
	 * 
	 * @param project_id
	 * @param secret
	 * @param from_id
	 * @param to_id
	 * @param graph_id
	 * @param rel_type
	 * @return
	 * @throws Exception
	 */
	private JSONArray queryRelations(String project_id, String secret, String from_id, String to_id, String graph_id,
                                     String rel_type) throws Exception {
		JSONObject jsonObject = new JSONObject();
		JSONObject criteria = new JSONObject();
		criteria.put("from_id", from_id);
		criteria.put("to_id", to_id);
		criteria.put("graph_id", graph_id);
		criteria.put("rel_type", rel_type);
		jsonObject.put("criteria", criteria);
		jsonObject.put("projectId", project_id);
//		String relationUrl = getDataPlatformPath(DataRequestPathUtil.dataPlat_relation_relation_instance_query,
//				project_id, secret);
		String relationUrl = getDataPlatformPath(DataRequestPathUtil.dataPlat_relation_relation_instance_query);
		JSONArray relations = null;
//		String queryResult = httpPostRequest(relationUrl, jsonObject.toJSONString());
		String queryResult = httpPostRequestDataPlatform(relationUrl, jsonObject.toJSONString());
		JSONObject relationJson = JSONObject.parseObject(queryResult);
		if (Result.SUCCESS.equals(relationJson.getString(Result.RESULT))) {
			relations = relationJson.getJSONArray(Result.CONTENT);
		}
		relations = relations == null ? new JSONArray() : relations;
		return relations;
	}

	private JSONArray getFloorIds(JSONObject confirmResult, String projectId) throws Exception {
		JSONArray floors = null;
		// 1.获取项目下所有楼层，并排序
		if (confirmResult.getJSONArray("scope") == null || confirmResult.getJSONArray("scope").isEmpty()) {
			// floors = subset_query(projectId, "Fl");
			floors = getAllFloor(projectId);
		} else {
			JSONObject requstJson = new JSONObject();
			JSONArray criterias = new JSONArray();
			JSONObject item;
			JSONArray spaceForScope = querySpacesByScope(confirmResult.getJSONArray("scope"), projectId);

			for (Object object : spaceForScope) {
				JSONObject space = (JSONObject) object;
				item = new JSONObject();
				String floorId = space.getJSONObject("location").getString("floor");
				if (!StringUtil.isNull(floorId)) {
					item.put("id", floorId);
					criterias.add(item);
				}
			}
			requstJson.put("criterias", criterias);
			requstJson.put("projectId", projectId);
			floors = batch_query(requstJson);
		}
		// key:sequenceId value:floorIds
		Map<String, JSONArray> sequenceIdAndFloorMap = getSequenceIdAndObjMap(floors);
		// 2.筛选楼层
		JSONArray resultFloorIds = filterBySequence(sequenceIdAndFloorMap, confirmResult);
		return resultFloorIds;
	}

	/**
	 * Description:
	 * 
	 * @param map
	 * @param count
	 * @param skip_count
	 * @return
	 * @throws NumberFormatException
	 *             JSONArray
	 * @author yuecaipu
	 * @since 2018年7月26日: 下午3:04:17 Update By yuecaipu 2018年7月26日: 下午3:04:17
	 */
	private JSONArray filterBySequence(Map<String, JSONArray> map, JSONObject confirmResult)
			throws NumberFormatException {

		if (map.isEmpty()) {
			return new JSONArray();
		}

		// 所有的序号集合
		List<String> completeSequenceIds = new ArrayList<>(map.keySet());
		int total = completeSequenceIds.size();// 序号总个数

		String unit = confirmResult.getString("unit");// 单位 1:个 2：%
		Integer count = Integer.parseInt(confirmResult.getString("count"));// 需要序号的个数
		if ("2".equals(unit)) {
			count = count * total / 100;
		}
		count = count > total ? total : count;
		Integer frequency = skipsCache.get().getInteger("frequency");
		frequency = frequency == null ? 0 : frequency;
		Integer startIndex = frequency * count % total;
		Integer endIndex = startIndex + count;
		endIndex = endIndex > total ? total : endIndex;
		// 需要的序号集合
		List<String> needSequenceIds = new ArrayList<>();
		for (int i = startIndex; i < endIndex; i++) {
			needSequenceIds.add(completeSequenceIds.get(i));
		}

		endIndex = startIndex + count - endIndex;
		for (int i = 0; i < endIndex; i++) {
			needSequenceIds.add(completeSequenceIds.get(i));
		}
		// 序号对应的返回id集合
		JSONArray resultIds = new JSONArray();
		for (String sequenceId : needSequenceIds) {
			resultIds.addAll(map.get(sequenceId));
		}

		return resultIds;
	}

	/**
	 * 
	 * Description: 对象序号和对象id对应map
	 * 
	 * @param objs
	 * @return Map<String,JSONArray>
	 * @author yuecaipu
	 * @since 2018年7月26日: 下午4:31:53 Update By yuecaipu 2018年7月26日: 下午4:31:53
	 */
	private Map<String, JSONArray> getSequenceIdAndObjMap(JSONArray objs) {
		Map<String, JSONArray> map = ToolsUtil.getAscMap();
		if (!objs.isEmpty()) {
			JSONArray objIds;
			JSONObject obj, infos;
			String objId;
			String sequenceId;
			// 序号信息点编码
			String sequenceCode = getSequenceCode(objs.getJSONObject(0).getString("id"));
			for (int i = 0; i < objs.size(); i++) {
				obj = objs.getJSONObject(i);
				objId = obj.getString("id");
				infos = obj.getJSONObject("infos");
				if (infos != null) {
					sequenceId = infos.getString(sequenceCode);
					if (!StringUtil.isNull(sequenceId)) {
						objIds = map.getOrDefault(sequenceId, new JSONArray());
						objIds.add(objId);
						map.put(sequenceId, objIds);
					}
				}
			}
		}
		return map;
	}

	/**
	 * Description: 根据id获取对应的顺序号信息点编码
	 * 
	 * @param sequenceCode
	 * @param id
	 * @return String
	 * @author yuecaipu
	 * @since 2018年7月26日: 下午4:42:58 Update By yuecaipu 2018年7月26日: 下午4:42:58
	 */

	private String getSequenceCode(String id) {
		String sequenceCode = null;
		if (id.startsWith("Sh")) {
			sequenceCode = "ShaftSequenceID";
		} else if (id.startsWith("Sp")) {
			sequenceCode = "RoomSerial";
		} else if (id.startsWith("Fl")) {
			sequenceCode = "FloorSequenceID";
		} else if (id.startsWith("Eq")) {
			sequenceCode = "EquipSerial";
		}
		return sequenceCode;
	}

	private JSONArray subset_query(String projectId, String type) throws Exception {

		String url = getDataPlatformPath(DataRequestPathUtil.dataPlat_object_subset_query);
//		String url = getDataPlatformPath(DataRequestPathUtil.dataPlat_object_subset_query, projectId, null);
//		String reusltStr = httpPostRequest(url, "{\"criteria\":{\"type\":[\"" + type + "\"]}}");
		String reusltStr = httpPostRequestDataPlatform(url, "{\"projectId\":\"" + projectId + "\",\"criteria\":{\"type\":[\"" + type + "\"]}}");
		JSONObject reusltObj = JSONObject.parseObject(reusltStr);
		JSONArray content = null;
		if (Result.SUCCESS.equals(reusltObj.getString(Result.RESULT))) {
			content = reusltObj.getJSONArray(Result.CONTENT);
		}
		content = content == null ? new JSONArray() : content;

		return content;
	}

	private JSONArray batch_query(JSONObject requstParam) throws Exception {
//		String secret = projectCache.getProjectSecretById(requstParam.getString("projectId"));
//		String url = getDataPlatformPath(DataRequestPathUtil.dataPlat_object_batch_query,
//				requstParam.getString("projectId"), secret);
		String project_id = requstParam.getString("projectId");
		requstParam.put("projectId", project_id);
		String url = getDataPlatformPath(DataRequestPathUtil.dataPlat_object_batch_query);
//		String reusltStr = httpPostRequest(url, requstParam.toJSONString());
		String reusltStr = httpPostRequestDataPlatform(url, requstParam.toJSONString());
		JSONObject reusltObj = JSONObject.parseObject(reusltStr);
		JSONArray content = null;
		if (Result.SUCCESS.equals(reusltObj.getString(Result.RESULT))) {
			content = reusltObj.getJSONArray(Result.CONTENT);
		}
		content = content == null ? new JSONArray() : content;

		return content;
	}

	private JSONArray query_by_info(JSONObject requstParam) throws Exception {
//		String secret = projectCache.getProjectSecretById(requstParam.getString("projectId"));
		String url = getDataPlatformPath(DataRequestPathUtil.dataPlat_object_query_by_info);
		String project_id = requstParam.getString("projectId");
		requstParam.put("projectId", project_id);
//		String reusltStr = httpPostRequest(url, requstParam.toJSONString());
		String reusltStr = httpPostRequestDataPlatform(url, requstParam.toJSONString());
		JSONObject reusltObj = JSONObject.parseObject(reusltStr);
		JSONArray content = null;
		if (Result.SUCCESS.equals(reusltObj.getString(Result.RESULT))) {
			content = reusltObj.getJSONArray(Result.CONTENT);
		}
		content = content == null ? new JSONArray() : content;

		return content;
	}
	
	private JSONArray relation_query_subset(String projectId, String graphId, JSONArray relTypes) throws Exception {

//		String url = getDataPlatformPath(DataRequestPathUtil.dataPlat_relation_query_subset, projectId, null);
		String url = getDataPlatformPath(DataRequestPathUtil.dataPlat_relation_query_subset);
		JSONObject param = new JSONObject();
		param.put("projectId", projectId);
		JSONObject criteria = new JSONObject();
		param.put("criteria", criteria);
		criteria.put("graph_id", graphId);
		criteria.put("rel_types", relTypes);
//		String reusltStr = httpPostRequest(url, param.toJSONString());
		String reusltStr = httpPostRequestDataPlatform(url, param.toJSONString());
		JSONObject reusltObj = JSONObject.parseObject(reusltStr);
		JSONArray content = null;
		if (Result.SUCCESS.equals(reusltObj.getString(Result.RESULT))) {
			content = reusltObj.getJSONArray(Result.CONTENT);
		}
		content = content == null ? new JSONArray() : content;

		return content;
	}
	private ThreadLocal<Map<String, String>> spaceRelationMapChache = new ThreadLocal<>();

	private Map<String, String> getSpaceRelation(String projectId) throws Exception {
		Map<String, String> spaceRelationMap = spaceRelationMapChache.get();
		if (spaceRelationMap == null) {
			spaceRelationMap = objectService.getSpaceRelation(projectId);
			spaceRelationMapChache.set(spaceRelationMap);
		}
		return spaceRelationMap;
	}

	private JSONArray filterEquipByScope(JSONArray scopes, JSONArray allInstanceEquipObj, String projectId)
			throws Exception {
		if (scopes == null || scopes.isEmpty()) {
			return allInstanceEquipObj;
		}

		JSONArray resultArray = new JSONArray();
		// key:equip_id value:space_id
		// Map<String, String> spaceRelationMap =
		// objectService.getSpaceRelation(projectId);
		Map<String, String> spaceRelationMap = getSpaceRelation(projectId);
		JSONArray spaceForScope = querySpacesByScope(scopes, projectId);
		JSONArray spaceIdsForScope = new JSONArray();
		JSONArray equipIdsForScope = new JSONArray();
		for (Object object : spaceForScope) {
			JSONObject space = (JSONObject) object;
			spaceIdsForScope.add(space.getString("id"));
		}

		Set<String> keySet = spaceRelationMap.keySet();
		for (String equipId : keySet) {
			if (spaceIdsForScope.contains(spaceRelationMap.get(equipId))) {
				equipIdsForScope.add(equipId);
			}
		}

		for (Object object : allInstanceEquipObj) {
			JSONObject equip = (JSONObject) object;
			if (equipIdsForScope.contains(equip.getString("obj_id"))) {
				resultArray.add(equip);
			}
		}

		return resultArray;
	}

	private JSONArray getEquipIds(JSONArray allInstanceObj, JSONObject confirmResult, String projectId)
			throws Exception {
		if (allInstanceObj != null) {
			JSONObject param = new JSONObject();
			JSONArray criterias = new JSONArray();
			param.put("projectId", projectId);
			param.put("criterias", criterias);
			JSONObject criteria;
			for (int i = 0; i < allInstanceObj.size(); i++) {
				JSONObject obj = allInstanceObj.getJSONObject(i);
				String objId = obj.getString("obj_id");
				criteria = new JSONObject();
				criteria.put("id", objId);
				criterias.add(criteria);
			}
			String url = getDataPlatformPath(DataRequestPathUtil.dataPlat_object_batch_query_post);
//			String resultStr = httpPostRequest(url, param.toJSONString());
			String resultStr = httpPostRequestDataPlatform(url, param.toJSONString());
			JSONObject resultObj = JSONObject.parseObject(resultStr);

			if (Result.SUCCESS.equals(resultObj.getString(Result.RESULT))) {
				JSONArray equips = resultObj.getJSONArray(Result.CONTENT);
				Map<String, JSONArray> sequenceIdAndSpaceMap = getSequenceIdAndObjMap(equips);
				// 筛选设备
				JSONArray result = filterBySequence(sequenceIdAndSpaceMap, confirmResult);
				return result;
			}
		}
		return new JSONArray();
	}

	private JSONArray getEquipIdsBySpaceIds(JSONArray spaceIds) throws Exception {
		JSONArray result = new JSONArray();
		if (spaceIds != null && spaceIds.size() > 0) {
			// 请求数据平台relation/query接口 获取图关系
			String projectId = "Pj" + spaceIds.getString(0).substring(2, 12);
			String graphId = graphCache.getGraphIdByProjectIdAndGraph(projectId, "EquipinSpace");
			JSONObject param = new JSONObject();
			JSONArray criterias = new JSONArray();
			param.put("projectId", projectId);
			param.put("merge", true);
			param.put("criterias", criterias);
			JSONObject criteria;
			for (int i = 0; i < spaceIds.size(); i++) {
				criteria = new JSONObject();
				criteria.put("graph_id", graphId);
				criteria.put("to_id", spaceIds.getString(i));
				criteria.put("rel_type", "1");
				criterias.add(criteria);
			}
//			String url = getDataPlatformPath(DataRequestPathUtil.dataPlat_relation_relation_instance_query, projectId,
//					null);
			String url = getDataPlatformPath(DataRequestPathUtil.dataPlat_relation_relation_instance_query);

//			String resultStr = httpPostRequest(url, param.toJSONString());
			String resultStr = httpPostRequestDataPlatform(url, param.toJSONString());
			JSONObject resultObj = JSONObject.parseObject(resultStr);
			if (Result.SUCCESS.equals(resultObj.getString(Result.RESULT))) {
				JSONArray content = resultObj.getJSONArray(Result.CONTENT);
				// 遍历结果 提取设备id
				for (int i = 0; i < content.size(); i++) {
					result.add(content.getJSONObject(i).getString("from_id"));
				}
			}
		}
		return result;
	}

	/**
	 * 实例化系统类对象
	 * 
	 * @param projectId
	 * @throws Exception
	 */
	private JSONArray instantiateSystemClass(JSONObject confirmResult, String projectId) throws Exception {
		return instanceAllObj(confirmResult, projectId);
	}

	/**
	 * 
	 * Description: 实例化全部对象
	 * 
	 * @param confirmResult
	 * @param projectId
	 * @return
	 * @throws Exception
	 *             JSONArray
	 * @author yuecaipu
	 * @since 2018年7月26日: 上午9:23:32 Update By yuecaipu 2018年7月26日: 上午9:23:32
	 */
	private JSONArray instanceAllObj(JSONObject confirmResult, String projectId) throws Exception {
		String info_points = confirmResult.getString("info_points");// 信息点
		String customs = confirmResult.getString("customs");// 自定义信息点
		String objId = confirmResult.getString("obj_id");
		String objType = confirmResult.getString("obj_type");
//		Map<String, String> spaceRelationMap = getSpaceRelation(projectId);
		// 获取项目下所有实例
//		JSONArray instantiateObjs = objectService.queryInstanceByObjType(projectId, objType, objId, spaceRelationMap);
		JSONArray instantiateObjs = queryInstanceByObjType(projectId, objType, objId);
		for (Object object : instantiateObjs) {
			JSONObject obj = (JSONObject) object;
			obj.put("customs", JSONObject.parseArray(customs));
			obj.put("info_points", JSONObject.parseArray(info_points));
		}
		return instantiateObjs;
	}	

	private JSONArray queryInstanceByObjType(String projectId, String objType, String objId) throws Exception {
		Map<String, String> id2name = getNameByids(projectId);
		JSONArray result = new JSONArray();
		if ("space_class".equals(objType)) {
			JSONArray allSpaces = getAllSpace(projectId);
			Map<String, List<JSONObject>> roomFuncType2space = assortSpaceByRoomFuncType(allSpaces);
			for (Entry<String, List<JSONObject>> entry : roomFuncType2space.entrySet()) {
				if (entry.getKey().equals(objId)) {
					List<JSONObject> spaces = entry.getValue();
					for (JSONObject space : spaces) {
						JSONObject infos = space.getJSONObject("infos");
						JSONObject item = new JSONObject();
						JSONObject parent = new JSONObject();
						JSONArray parents = new JSONArray();
						item.put("obj_id", space.getString("id"));
						item.put("obj_name", infos.getString("RoomLocalName"));
						item.put("obj_type", "space");
						item.put("RoomSerial", infos.getString("RoomSerial"));
						JSONArray parent_ids = new JSONArray();
						JSONObject location = space.getJSONObject("location");
						parent_ids.addAll(location.values());
						parent.put("parent_ids", parent_ids);
						JSONArray parent_names = new JSONArray();
						for (Object object : parent_ids) {
							parent_names.add(id2name.get(object));
						}
						parent.put("parent_names", parent_names);
						parents.add(parent);
						item.put("parents", parents);
						result.add(item);
					}
				}
			}
		}else if ("system_class".equals(objType)) {
			JSONArray systems = subset_query(projectId, objId);
			for (Object object : systems) {
				JSONObject system = (JSONObject)object;
				JSONObject infos = system.getJSONObject("infos");
				JSONObject item = new JSONObject();
				JSONObject parent = new JSONObject();
				JSONArray parents = new JSONArray();
				item.put("obj_id", system.getString("id"));
				item.put("obj_name", infos.getString("SysLocalName"));
				item.put("obj_type", "system");
				JSONArray parent_ids = new JSONArray();
				JSONObject location = system.getJSONObject("location");
				parent_ids.addAll(location.values());
				parent.put("parent_ids", parent_ids);
				JSONArray parent_names = new JSONArray();
				for (Object object2 : parent_ids) {
					parent_names.add(id2name.get(object2));
				}
				parent.put("parent_names", parent_names);
				parents.add(parent);
				item.put("parents", parents);
				result.add(item);		
			}
		}else if ("equip_class".equals(objType)) {
			JSONArray equips = subset_query(projectId, objId);
			for (Object object : equips) {
				JSONObject equip = (JSONObject)object;
				JSONObject infos = equip.getJSONObject("infos");
				JSONObject item = new JSONObject();
				JSONObject parent = new JSONObject();
				JSONArray parents = new JSONArray();
				item.put("obj_id", equip.getString("id"));
				item.put("obj_name", infos.getString("EquipLocalName"));
				item.put("obj_type", "equip");
				JSONArray parent_ids = new JSONArray();
				JSONObject location = equip.getJSONObject("location");
				parent_ids.addAll(location.values());
				JSONArray parent_names = new JSONArray();
				for (Object object2 : parent_ids) {
					parent_names.add(id2name.get(object2));
				}
				parent.put("parent_ids", parent_ids);
				parent.put("parent_names", parent_names);
				parents.add(parent);
				item.put("parents", parents);
				result.add(item);		
			}
		}
		return result;
	}

	private Map<String, String> getNameByids(String projectId) throws Exception {
		Map<String, String> map = new HashMap<>();
//		JSONArray shafts = getAllShaft(projectId);
//		JSONArray spaces = getAllSpace(projectId);
		JSONArray builds = getAllBuild(projectId);
		JSONArray floors = getAllFloor(projectId);
		for (Object object : builds) {
			JSONObject build = (JSONObject)object;
			map.put(build.getString("id"), build.getJSONObject("infos").getString("BuildLocalName"));
		}
		for (Object object : floors) {
			JSONObject floor = (JSONObject)object;
			map.put(floor.getString("id"), floor.getJSONObject("infos").getString("FloorLocalName"));
		}
		return map;
	}
}

